package rexsee.baidu;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.util.EntityUtils;

import rexsee.core.application.RexseeApplication;
import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.JavascriptInterface;
import rexsee.core.browser.clazz.LoadingProgressListeners;
import rexsee.core.lang.RexseeLanguage;
import rexsee.core.transportation.RexseeUpload;
import rexsee.core.transportation.SecureHttpClient;
import rexsee.dialog.PopupWindow;
import android.app.Activity;
import android.content.Context;
import android.content.res.Resources;
import android.view.View;
import android.webkit.CookieManager;
import android.widget.Button;
import android.widget.LinearLayout;

public class RexseeBaiduPCS implements JavascriptInterface {

	public static final String INTERFACE_NAME = "BaiduPCS";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return new RexseeBaiduPCS(childBrowser);
	}

	public static final String URL_VERIFY = "https://openapi.baidu.com/oauth/2.0/authorize";

	public static final String EVENT_BAIDUPCSREADY = "onBaiduPCSReady";
	public static final String STRING_NAME = "ٶƴ洢";
	private String mKey;
	private String mSecret;
	private final Browser mBrowser;
	private final Context mContext;

	public static final int DEFAULT_TIMEOUT = 20000;
	private String mAccessToken;

	public RexseeBaiduPCS(Browser browser) {

		mBrowser = browser;
		mContext = mBrowser.getContext();

		mBrowser.eventList.add(EVENT_BAIDUPCSREADY);

		Resources r = browser.getContext().getResources();
		String packageName = browser.getContext().getPackageName();
		int id = r.getIdentifier("BaiduPCSKey", "string", packageName);
		mKey = (id == 0) ? null : r.getString(id);
		id = r.getIdentifier("BaiduPCSSecret", "string", packageName);
		mSecret = (id == 0) ? null : r.getString(id);

		mAccessToken = null;

	}

	private boolean _parseToken(String url) {
		//mBrowser.application.alert(url + "<br><br>" + mAccessToken);
		if (url != null && url.contains("access_token=")) {
			String tmp = url.split("access_token=")[1];
			if (tmp.contains("&")) tmp = tmp.substring(0, tmp.indexOf("&"));
			mAccessToken = URLDecoder.decode(tmp);
			mBrowser.eventList.run(EVENT_BAIDUPCSREADY);
			return true;
		} else return false;
	}

	//JavaScript Interface

	//Overrides
	public void setKeyAndSecret(String key, String secret) {
		mKey = key;
		mSecret = secret;
	}
	public String getKeyAndSecret() {
		return "{\"key\":\"" + mKey + "\",\"secret\":\"" + mSecret + "\",\"accessToken\":\"" + mAccessToken + "\"}";
	}
	public void login() {
		//logout();
		((Activity) mContext).runOnUiThread(new Runnable() {
			@Override
			public void run() {
				try {
					String url = URL_VERIFY;
					url += "?response_type=token";
					url += "&client_id=" + mKey;
					url += "&redirect_uri=oob";
					url += "&scope=basic,netdisk";
					url += "&display=mobile";
					final PopupWindow mVerifierWindow = new PopupWindow(mBrowser, "BaiduPCSVerifierWindow", url, "window-cancelable:false;", null);
					mVerifierWindow.mChild.allowErrorDialog = false;
					/*
					mVerifierWindow.mChild.pageBeforeStartListeners.add(new PageBeforeStartListener() {
						@Override
						public void run(Browser browser, String url) {
							if (_parseToken(url)) {
								mVerifierWindow.mChild.stopLoading();
								mVerifierWindow.dismiss();
							}
						}
					});
					*/
					mVerifierWindow.mChild.progressListeners.add(new LoadingProgressListeners.LoadingProgressListener() {
						@Override
						public void run(final Context context, Browser browser, int progress) {
							if (progress == 100 && _parseToken(browser.getUrl())) {
								mVerifierWindow.mChild.stopLoading();
								mVerifierWindow.dismiss();
							}
						}
					});

					Button cancel = new Button(mBrowser.getContext());
					cancel.setText(android.R.string.cancel);
					cancel.setOnClickListener(new View.OnClickListener() {
						@Override
						public void onClick(View v) {
							mVerifierWindow.dismiss();
						}
					});
					cancel.setLayoutParams(new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, 70, 1));
					mVerifierWindow.mBottomLayout.setOrientation(LinearLayout.HORIZONTAL);
					mVerifierWindow.mBottomLayout.setPadding(0, 5, 0, 0);
					mVerifierWindow.mBottomLayout.addView(cancel);
					mVerifierWindow.start();
				} catch (Exception e) {
					mBrowser.exception(getInterfaceName(), e);
				}
			}
		});
	}
	public void logout() {
		CookieManager cookieManager = CookieManager.getInstance();
		cookieManager.removeAllCookie();
		mAccessToken = null;
	}
	public boolean isReady() {
		return mAccessToken != null;
	}

	//GET
	public String quota() {
		mBrowser.progressDialog.show(RexseeLanguage.PROGRESS_WAITING_FOR_SERVER);
		String url = "https://pcs.baidu.com/rest/2.0/pcs/quota";
		url += "?method=info";
		url += "&access_token=" + mAccessToken;
		return _get(url);
	}
	public String list(String path, int byInt, boolean increase) {
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=list";
		url += "&access_token=" + mAccessToken;
		url += "&path=" + path;
		url += "&by=" + ((byInt == 3) ? "size" : ((byInt == 2) ? "time" : "name"));
		url += "&order=" + ((increase) ? "asc" : "desc");
		return _get(url);
	}
	public String search(String path, String keyword, boolean recursive) {
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=search";
		url += "&access_token=" + mAccessToken;
		url += "&path=" + path;
		url += "&wd=" + keyword;
		url += "&re=" + ((recursive) ? "1" : "0");
		return _get(url);
	}
	public String meta(String path) {
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=meta";
		url += "&access_token=" + mAccessToken;
		url += "&path=" + path;
		return _get(url);
	}
	public String mkdir(String path) {
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=mkdir";
		url += "&access_token=" + mAccessToken;
		url += "&path=" + path;
		return _get(url);
	}
	public String remove(String path) {
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=delete";
		url += "&access_token=" + mAccessToken;
		url += "&path=" + path;
		return _get(url);
	}
	public String move(String from, String to) {
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=move";
		url += "&access_token=" + mAccessToken;
		url += "&from=" + from;
		url += "&to=" + to;
		return _get(url);
	}
	public String copy(String from, String to) {
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=copy";
		url += "&access_token=" + mAccessToken;
		url += "&from=" + from;
		url += "&to=" + to;
		return _get(url);
	}

	private String _get(String url) {
		if (mAccessToken == null) {
			mBrowser.exception(getInterfaceName() + "._checkToken", "Access token is null.");
			return "";
		} else {
			mBrowser.progressDialog.show(RexseeLanguage.PROGRESS_WAITING_FOR_SERVER);
			String rtn = new SecureHttpClient(mBrowser).get(url);
			mBrowser.progressDialog.hide();
			return rtn;
		}
	}

	//POST
	private String syncResult = null;

	public void download(String path, String localPath) {
		if (mAccessToken == null) {
			mBrowser.exception(getInterfaceName() + "._checkToken", "Access token is null.");
			//return RexseeApplication.EXCEPTION_PREFIX;
			return;
		}
		if (localPath.toLowerCase().startsWith("file://")) localPath = localPath.substring(7);
		final String file = localPath;
		String url = "https://pcs.baidu.com/rest/2.0/pcs/file";
		url += "?method=download";
		url += "&access_token=" + mAccessToken;
		url += "&path=" + path;
		final String finalUrl = url;
		syncResult = null;
		mBrowser.progressDialog.show(RexseeLanguage.PROGRESS_DOWNLOAD + "......");
		new Thread() {
			@Override
			public void run() {
				HttpClient httpClient = new SecureHttpClient(mBrowser).getNewHttpClient(mContext, DEFAULT_TIMEOUT, DEFAULT_TIMEOUT);;
				final HttpGet request = new HttpGet(finalUrl);
				final HttpResponse resp;
				try {
					resp = httpClient.execute(request);
					if (resp.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
						FileOutputStream out = new FileOutputStream(file);
						resp.getEntity().writeTo(out);
						out.close();
						mBrowser.progressDialog.hide();
						syncResult = "";
					} else {
						mBrowser.progressDialog.hide();
						syncResult = EntityUtils.toString(resp.getEntity());
					}
				} catch (final IOException e) {
					mBrowser.progressDialog.hide();
					mBrowser.exception(getInterfaceName(), e);
				}
			}
		}.start();
		while (syncResult == null) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
			}
		}
		mBrowser.progressDialog.hide();
	}
	public String upload(String path, String localPath) {
		if (mAccessToken == null) {
			mBrowser.exception(getInterfaceName() + "._checkToken", "Access token is null.");
			return RexseeApplication.EXCEPTION_PREFIX;
		}
		String uploadUrl = "https://pcs.baidu.com/rest/2.0/pcs/file";
		uploadUrl += "?method=upload";
		uploadUrl += "&access_token=" + mAccessToken;
		uploadUrl += "&path=" + path;
		final String finalUrl = uploadUrl;
		final String filename = localPath.substring(localPath.lastIndexOf("/") + 1);
		if (localPath.toLowerCase().startsWith("file://")) localPath = localPath.substring(7);
		final String file = localPath;
		mBrowser.progressDialog.show(RexseeLanguage.PROGRESS_UPLOAD + "......");
		syncResult = null;
		new Thread() {
			@Override
			public void run() {
				try {
					URL url = new URL(finalUrl);
					HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
					urlConnection.setRequestMethod("POST");
					urlConnection.setDoInput(true);
					urlConnection.setDoOutput(true);
					urlConnection.setConnectTimeout(DEFAULT_TIMEOUT);
					urlConnection.setReadTimeout(DEFAULT_TIMEOUT);
					urlConnection.setUseCaches(false);
					//urlConnection.setRequestProperty("User-Agent", mBrowser.getSettings().getUserAgentString());
					urlConnection.setRequestProperty("Connection", "Keep-Alive");
					urlConnection.setRequestProperty("Content-Type", "multipart/form-data;boundary=" + RexseeUpload.BOUNDARY);
					urlConnection.connect();
					DataOutputStream dos = new DataOutputStream(urlConnection.getOutputStream());
					dos.writeBytes(RexseeUpload.DOUBLEHYPHENS + RexseeUpload.BOUNDARY + RexseeUpload.LINEEND);
					dos.writeBytes("Content-Disposition: form-data; name=\"file\"; filename=\"" + filename + "\"; " + RexseeUpload.LINEEND);
					dos.writeBytes(RexseeUpload.LINEEND);
					File theFile = new File(file);
					long totalLength = theFile.length();
					FileInputStream fileInputStream = new FileInputStream(theFile);
					int uploadedSize = 0;
					byte[] buffer = new byte[1024];
					int bufferLength = 0;
					while ((bufferLength = fileInputStream.read(buffer)) > 0) {
						dos.write(buffer, 0, bufferLength);
						uploadedSize += bufferLength;
						int progress = (int) (uploadedSize * 100 / totalLength);
						mBrowser.progressDialog.show(RexseeLanguage.PROGRESS_UPLOAD + "......" + progress + "%");
					}
					dos.writeBytes(RexseeUpload.LINEEND);
					dos.writeBytes(RexseeUpload.DOUBLEHYPHENS + RexseeUpload.BOUNDARY + RexseeUpload.DOUBLEHYPHENS + RexseeUpload.LINEEND);
					fileInputStream.close();
					dos.flush();
					dos.close();

					InputStream responseStream = urlConnection.getInputStream();
					InputStreamReader responseStreamReader = new InputStreamReader(responseStream);
					BufferedReader bufReader = new BufferedReader(responseStreamReader);
					String line = null;
					String response = "";
					while ((line = bufReader.readLine()) != null) {
						response += line;
					}
					responseStream.close();
					urlConnection.disconnect();
					mBrowser.progressDialog.hide();
					syncResult = response;

				} catch (Exception e) {
					mBrowser.progressDialog.hide();
					mBrowser.exception(getInterfaceName() + ".upload", e.getLocalizedMessage() + "[" + finalUrl + "]");
					syncResult = "";
				}
			}
		}.start();
		while (syncResult == null) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
			}
		}
		mBrowser.progressDialog.hide();
		return syncResult;
	}

}
