package rexsee.content;

import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.JavascriptInterface;
import rexsee.core.browser.clazz.UrlListener;
import rexsee.core.lang.RexseeLanguage;
import rexsee.core.utilities.Utilities;
import android.content.ContentResolver;
import android.content.Context;
import android.database.Cursor;
import android.provider.CallLog.Calls;

public class RexseeCallLog implements JavascriptInterface {

	public static final String INTERFACE_NAME = "CallLog";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return new RexseeCallLog(childBrowser);
	}

	private static final String[] CALLS_COLUMNS = new String[]{
			Calls.NUMBER, //0
			Calls.TYPE, //1
			Calls.DATE, //2
			Calls.DURATION //3
	};

	private final Context mContext;
	private final Browser mBrowser;

	public RexseeCallLog(Browser browser) {
		mBrowser = browser;
		mContext = browser.getContext();
		browser.urlListeners.add(new UrlListener(mBrowser.application.resources.prefix + ":callLog") {
			@Override
			public void run(final Context context, final Browser browser, String url) {
				new Thread() {
					@Override
					public void run() {
						browser.progressDialog.show(RexseeLanguage.PROGRESS_ONGOING);
						String html = "<HTML><HEAD><TITLE>" + Calls.CONTENT_URI.toString() + "</TITLE></HEAD>";
						html += "<BODY style='margin:0px;background-color:black;color:white;'>";
						html += "<table width=100% cellspacing=0 style='color:white;font-size:14px;'>";
						try {
							ContentResolver contentResolver = context.getContentResolver();
							String[] columns = new String[]{
									Calls.NUMBER, //0
									Calls.TYPE, //1
									Calls.DATE, //2
									Calls.DURATION //3
							};
							Cursor cursor = contentResolver.query(Calls.CONTENT_URI, columns, null, null, Calls.DATE + " DESC limit 100");
							if (cursor == null || cursor.getCount() == 0) {
								html += "<tr><td style='padding:10px;'>No recent call found.</td></tr>";
							} else {
								for (int i = 0; i < cursor.getCount(); i++) {
									cursor.moveToPosition(i);
									html += "<tr><td style='border-bottom:1px solid #222222; padding:10 5 10 5;'>";
									html += "<div style='font-size:16px;font-weight:bold;'>" + cursor.getString(0) + "</div>";
									html += "<div>" + getType(cursor.getInt(1)) + "</div>";
									html += "<div>" + Utilities.timeStamp2dateString(cursor.getLong(2)) + "</div>";
									html += "<div>" + cursor.getString(3) + " seconds</div>";
									html += "</td></tr>";
								}
							}
							cursor.close();
						} catch (Exception e) {
							html += "<tr><td style='padding:10px;'>Exception: " + e.getLocalizedMessage() + "</td></tr>";
						}
						html += "</table>";
						html += "</BODY>";
						html += "</HTML>";
						browser.function.loadHTMLWithoutHistory(html);
					}
				}.start();
			}
			@Override
			public boolean shouldAddToHistory(Context context, Browser browser, String url) {
				return true;
			}
		});
	}

	private static String getType(int type) {
		switch (type) {
			case Calls.MISSED_TYPE :
				return "MISSED";
			case Calls.INCOMING_TYPE :
				return "INCOMING";
			case Calls.OUTGOING_TYPE :
				return "OUTGOING";
			default :
				return "unknown";
		}
	}
	private static int getType(String type) {
		if (type == null) return -1;
		else if (type.equalsIgnoreCase("MISSED")) return Calls.MISSED_TYPE;
		else if (type.equalsIgnoreCase("INCOMING")) return Calls.INCOMING_TYPE;
		else if (type.equalsIgnoreCase("OUTGOING")) return Calls.OUTGOING_TYPE;
		else return -1;
	}

	//JavaScript Interface

	public String getContentUri() {
		return Calls.CONTENT_URI.toString();
	}
	public String get(int number) {
		return get(null, number);
	}
	public String get(String type, int number) {
		int t = getType(type);
		String selection = (t > 0) ? Calls.TYPE + "=" + t : null;
		return getData(selection, number);
	}
	public String getData(String selection, int number) {
		ContentResolver contentResolver = mContext.getContentResolver();
		Cursor cursor;
		if (number > 0) {
			cursor = contentResolver.query(Calls.CONTENT_URI, CALLS_COLUMNS, selection, null, Calls.DATE + " DESC limit " + number);
		} else {
			cursor = contentResolver.query(Calls.CONTENT_URI, CALLS_COLUMNS, selection, null, Calls.DATE + " DESC");
		}
		if (cursor == null) return "[]";
		String rtn = "";
		for (int i = 0; i < cursor.getCount(); i++) {
			cursor.moveToPosition(i);
			if (i > 0) rtn += ",";
			rtn += "{";
			rtn += "\"number\":\"" + cursor.getString(0) + "\"";
			rtn += ",\"type\":\"" + getType(cursor.getInt(1)) + "\"";
			rtn += ",\"date\":" + cursor.getString(2);
			rtn += ",\"duration\":" + cursor.getString(3);
			rtn += "}";
		}
		cursor.close();
		return "[" + rtn + "]";
	}
	public String getColumns() {
		return RexseeContent.getContentColumns_(mContext, Calls.CONTENT_URI);
	}

}
