package rexsee.core.layout;

import rexsee.core.browser.Browser;
import rexsee.core.browser.BrowserDiv;
import rexsee.core.browser.clazz.LoadResourceListeners;
import rexsee.core.browser.clazz.LoadingProgressListeners;
import rexsee.core.browser.clazz.ReceivedTitleListeners;
import rexsee.core.browser.clazz.RootViewInterface;
import rexsee.core.lang.RexseeLanguage;
import rexsee.core.widget.Div;
import rexsee.core.widget.LabelDiv;
import rexsee.core.widget.ProgressDiv;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Rect;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.LinearLayout;

public class Layout extends Div implements RootViewInterface {

	public final LinearLayout layout = new LinearLayout(this.getContext());
	public final LinearLayout top = new LinearLayout(this.getContext());
	public final LinearLayout bottom = new LinearLayout(this.getContext());
	public final LinearLayout middle = new LinearLayout(this.getContext());
	public final FrameLayout left = new FrameLayout(this.getContext());
	public final FrameLayout right = new FrameLayout(this.getContext());

	public final BrowserDiv browserDiv = new BrowserDiv(this.getContext(), null, null);

	public final LabelDiv titleBar = new LabelDiv(this.getContext());
	public final ProgressDiv statusBar = new ProgressDiv(this.getContext());
	public boolean statusBarAutoHide = false;

	public final Browser browser;

	public Layout(Context context) {

		super(context);
		browser = browserDiv.browser;

		layout.setLayoutParams(new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, FrameLayout.LayoutParams.FILL_PARENT));
		layout.setOrientation(LinearLayout.VERTICAL);
		layout.setBackgroundColor(Color.TRANSPARENT);

		top.setOrientation(LinearLayout.VERTICAL);
		top.setTag("top");
		bottom.setOrientation(LinearLayout.VERTICAL);
		bottom.setTag("bottom");
		middle.setOrientation(LinearLayout.HORIZONTAL);
		browserDiv.setStyle("width:fillparent;height:fillparent;weight:2");
		middle.setTag("middle");
		left.setTag("left");
		right.setTag("right");
		middle.addView(left, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.FILL_PARENT));
		middle.addView(browserDiv);
		middle.addView(right, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.FILL_PARENT));

		String titleBarVisibility = (browser.application.resources.titleBar) ? "visible" : "hidden";
		titleBar.styleSheet.icon = browser.application.icon;
		titleBar.setStyle("visibility:" + titleBarVisibility + ";label:" + browser.application.label + ";height:30;width:fillparent;background-color:#888888;text-align:left;vertical-align:middle;text-link:false;white-space:nowrap;icon-position:left;label-padding-left:3px;border-width:0 0 1 0;border-color:#222222;padding:1 5 0 5;font-weight:bold;text-shadow:1.2 2 2 #000000;");
		String statusBarVisibility = (browser.application.resources.statusBar) ? "visible" : "hidden";
		statusBar.setStyle("visibility:" + statusBarVisibility + ";height:25px;progress_color:#888888;progress-text-color:#FFFFFF;background-color:#444444;color:#FFFFFF;");

		layout.addView(titleBar);
		layout.addView(top, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT));
		layout.addView(middle, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.FILL_PARENT, 1));
		layout.addView(bottom, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT));
		layout.addView(statusBar);

		browser.titleListeners.add(new ReceivedTitleListeners.ReceivedTitleListener() {
			@Override
			public void run(Context context, Browser browser, String title) {
				titleBar.setStyle("label:" + title + "&nbsp;");
			}
		});
		browser.progressListeners.add(new LoadingProgressListeners.LoadingProgressListener() {
			@Override
			public void run(Context context, Browser browser, int progress) {
				statusBar.setStyle("progress:" + progress + ";");
				if (statusBarAutoHide) statusBar.setVisibility(View.VISIBLE);
				if (progress == 100) {
					statusBar.setStyle("label:" + RexseeLanguage.PROGRESS_FINISH + ";");
					if (statusBarAutoHide) statusBar.setVisibility(View.GONE);
				}
			}
		});
		browser.loadResourceListeners.add(new LoadResourceListeners.LoadResourceListener() {
			@Override
			public void run(Context context, Browser browser, String url) {
				statusBar.setStyle("label:" + url + ";");
				if (statusBarAutoHide) statusBar.setVisibility(View.VISIBLE);
			}
		});

		this.setStyle("border-width:0;");
		this.addChild(layout);

	}

	@Override
	public void setStyle(String style) {
		styleSheet.parseStyle(style);
		setDivStyleWithoutLayoutParams();
	}

	@Override
	public View getView() {
		return this;
	}
	@Override
	public Browser getBrowser() {
		return browser;
	}
	@Override
	public Rect getRect() {
		Rect rect = new Rect();
		this.getWindowVisibleDisplayFrame(rect);
		return rect;
	}
	@Override
	public void addChild(String position, int index, View child) {
		if (position.equalsIgnoreCase("bottom")) {
			if (index >= 0) {
				bottom.addView(child, index);
			} else {
				bottom.addView(child);
			}
		} else if (position.equalsIgnoreCase("top")) {
			if (index >= 0) {
				top.addView(child, index);
			} else {
				top.addView(child);
			}
		} else if (position.equalsIgnoreCase("middle")) {
			if (index >= 0) {
				middle.addView(child, index);
			} else {
				middle.addView(child);
			}
		} else if (position.equalsIgnoreCase("left")) left.addView(child);
		else if (position.equalsIgnoreCase("right")) right.addView(child);
		else if (position.equalsIgnoreCase("browser")) browserDiv.addView(child);
		else if (position.equalsIgnoreCase("layout")) innerLayout.addView(child);
	}
	@Override
	public void removeChild(String position, View child) {
		if (position.equalsIgnoreCase("bottom")) {
			bottom.removeView(child);
			bottom.invalidate();
		} else if (position.equalsIgnoreCase("top")) {
			top.removeView(child);
			top.invalidate();
		} else if (position.equalsIgnoreCase("middle")) {
			middle.removeView(child);
			middle.invalidate();
		} else if (position.equalsIgnoreCase("left")) {
			left.removeView(child);
			left.invalidate();
		} else if (position.equalsIgnoreCase("right")) {
			right.removeView(child);
			right.invalidate();
		} else if (position.equalsIgnoreCase("browser")) {
			browserDiv.removeView(child);
			browserDiv.invalidate();
		} else if (position.equalsIgnoreCase("layout")) {
			innerLayout.removeView(child);
			innerLayout.invalidate();
		}
	}
	@Override
	public void startAnimation(final String style) {
		animate(browser, style);
	}
	@Override
	public void stopAnimation() {
		stopAnimate();
	}

}
