package rexsee.core.layout;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.JavascriptInterface;
import rexsee.core.browser.clazz.PageStartedListeners;
import rexsee.core.browser.clazz.RootViewInterface;
import rexsee.core.menu.PopMenu;
import rexsee.core.utilities.Utilities;
import rexsee.core.widget.ButtonBar;
import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;

public class RexseeButtonBars implements JavascriptInterface {

	public static final String INTERFACE_NAME = "ButtonBars";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return null;
	}

	private final Activity mActivity;
	private final RootViewInterface mLayout;
	private final Browser mBrowser;
	private final HashMap<String, ButtonBar> mMap;
	private boolean syncTag = false;

	public RexseeButtonBars(RootViewInterface layout) {
		super();
		mLayout = layout;
		mBrowser = layout.getBrowser();
		mActivity = (Activity) mBrowser.getContext();
		mMap = new HashMap<String, ButtonBar>();
		mBrowser.pageStartedListeners.add(new PageStartedListeners.PageStartedListener() {
			@Override
			public void run(Context context, Browser browser, final String url, Bitmap favicon) {
				((Activity) context).runOnUiThread(new Runnable() {
					@Override
					public void run() {
						remove();
					}
				});
			}
		});
	}

	public boolean exists(String id) {
		return mMap.containsKey(id);
	}
	public String getIds() {
		String rtn = "";
		Set<String> ids = mMap.keySet();
		Iterator<String> iterator = ids.iterator();
		while (iterator.hasNext()) {
			if (!rtn.equals("")) rtn += ",";
			rtn += "\"" + iterator.next() + "\"";
		}
		return "[" + rtn + "]";
	}
	public int size() {
		return mMap.size();
	}
	public void create(final String menuId) {
		if (menuId == null || menuId.equals("") || !mBrowser.menu.exists(menuId)) return;
		remove(menuId);
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				PopMenu menu = mBrowser.menu.menus.get(menuId);
				ButtonBar bar = new ButtonBar(mActivity, menu);
				mLayout.addChild(bar.styleSheet.bar_position, Utilities.getInt(bar.styleSheet.index, -1), bar);
				mMap.put(menuId, bar);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
		return;
	}
	public void remove() {
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Set<String> keys = mMap.keySet();
				Iterator<String> iterator = keys.iterator();
				while (iterator.hasNext()) {
					ButtonBar buttonBar = mMap.get(iterator.next());
					mLayout.removeChild(buttonBar.styleSheet.bar_position, buttonBar);
				}
				mMap.clear();
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}
	public void remove(final String id) {
		final ButtonBar buttonBar = mMap.get(id);
		if (buttonBar == null) return;
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				mLayout.removeChild(buttonBar.styleSheet.bar_position, buttonBar);
				mMap.remove(id);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}

	public void setStyle(String id, final String style) {
		final ButtonBar buttonBar = mMap.get(id);
		if (buttonBar == null) return;
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				mLayout.removeChild(buttonBar.styleSheet.bar_position, buttonBar);
				buttonBar.setBarStyle(style);
				mLayout.addChild(buttonBar.styleSheet.bar_position, Utilities.getInt(buttonBar.styleSheet.index, -1), buttonBar);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}
	public String getStyle(String id, String name) {
		final ButtonBar buttonBar = mMap.get(id);
		if (buttonBar == null) return "";
		String value = buttonBar.styleSheet.getStyle(name);
		return (value == null) ? "" : value;
	}
	public String getCurrentStyle(String id, String name) {
		final ButtonBar buttonBar = mMap.get(id);
		if (buttonBar == null) return "";
		String value = buttonBar.getStyle(name);
		return (value == null) ? "" : value;
	}

	public void startAnimation(String id, String style) {
		final ButtonBar bar = mMap.get(id);
		if (bar == null) return;
		bar.animate(mBrowser, style);
	}
	public void stopAnimation(String id) {
		final ButtonBar bar = mMap.get(id);
		if (bar == null) return;
		bar.stopAnimate();
	}

}
