package rexsee.core.layout;

import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.JavascriptInterface;
import rexsee.core.browser.clazz.RootViewInterface;
import android.app.Activity;

public class RexseeLayoutStyle implements JavascriptInterface {

	public static final String INTERFACE_NAME = "LayoutStyle";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return null;
	}

	private final Activity mActivity;
	private final Browser mBrowser;
	private final RootViewInterface mLayout;
	public RexseeLayoutStyle(RootViewInterface layout) {
		super();
		mLayout = layout;
		mBrowser = layout.getBrowser();
		mActivity = (Activity) mBrowser.getContext();
	}
	private boolean syncTag = false;

	public void setStyle(final String style) {
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				mLayout.setStyle(style);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}
	public String getStyle(String name) {
		String value = mLayout.getStyleSheet().getStyle(name);
		return (value == null) ? "" : value;
	}
	public String getCurrentStyle(String name) {
		String value = mLayout.getStyle(name);
		return (value == null) ? "" : value;
	}

	public void startAnimation(final String style) {
		mLayout.startAnimation(style);
	}
	public void stopAnimation() {
		mLayout.stopAnimation();
	}

}
