package rexsee.core.layout;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.HistoryItem;
import rexsee.core.browser.clazz.JavascriptInterface;
import rexsee.core.browser.clazz.ReceivedTitleListeners;
import rexsee.core.browser.clazz.RootViewInterface;
import rexsee.core.menu.PopMenu;
import rexsee.core.menu.PopMenuItem;
import rexsee.core.utilities.Utilities;
import rexsee.core.widget.Div;
import rexsee.core.widget.Tab;
import rexsee.core.widget.TabBar;
import android.app.Activity;
import android.content.Context;
import android.graphics.Rect;

public class RexseeTabBars implements JavascriptInterface {

	public static final String INTERFACE_NAME = "TabBars";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return null;
	}

	private final Activity mActivity;
	private final RootViewInterface mLayout;
	private final Browser mBrowser;
	private final HashMap<String, TabBar> mMap;
	private boolean syncTag = false;

	public RexseeTabBars(RootViewInterface layout) {
		super();
		mLayout = layout;
		mBrowser = layout.getBrowser();
		mActivity = (Activity) mBrowser.getContext();
		mMap = new HashMap<String, TabBar>();
		/*
		mBrowser.pageReadyListeners.add(new PageReadyListeners.PageReadyListener() {
			@Override
			public void run(Context context, Browser browser, final String url) {
				((Activity) context).runOnUiThread(new Runnable() {
					@Override
					public void run() {
						Set<String> keys = mMap.keySet();
						Iterator<String> iterator = keys.iterator();
						while (iterator.hasNext()) {
							mMap.get(iterator.next()).selectTab(url);
						}
					}
				});
			}
		});
		*/
		mBrowser.titleListeners.add(new ReceivedTitleListeners.ReceivedTitleListener() {
			@Override
			public void run(Context context, final Browser browser, final String t) {
				((Activity) context).runOnUiThread(new Runnable() {
					@Override
					public void run() {
						String url = browser.history.getCurrentItem().getUrl();
						Set<String> keys = mMap.keySet();
						Iterator<String> iterator = keys.iterator();
						while (iterator.hasNext()) {
							mMap.get(iterator.next()).selectTab(url);
						}
					}
				});
			}
		});
	}

	public HashMap<String, TabBar> getMap() {
		return mMap;
	}

	public boolean exists(String id) {
		return mMap.containsKey(id);
	}
	public String getIds() {
		String rtn = "";
		Set<String> ids = mMap.keySet();
		Iterator<String> iterator = ids.iterator();
		while (iterator.hasNext()) {
			if (!rtn.equals("")) rtn += ",";
			rtn += "\"" + iterator.next() + "\"";
		}
		return "[" + rtn + "]";
	}
	public int size() {
		return mMap.size();
	}
	public void create(final String menuId) {
		try {
			HistoryItem historyItem = mBrowser.history.getCurrentItem();
			if (historyItem != null) {
				create(menuId, historyItem.getUrl());
			} else {
				create(menuId, null);
			}
		} catch (Exception e) {
			mBrowser.exception(getInterfaceName(), e);
		}
	}
	public void create(final String menuId, final String selectedTabIds) {
		if (menuId == null || menuId.equals("") || !mBrowser.menu.exists(menuId)) return;
		remove(menuId);
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				try {
					PopMenu menu = mBrowser.menu.menus.get(menuId);
					final TabBar newBar = new TabBar(mActivity, menu);
					for (int i = 0; i < newBar.mContainer.getChildCount(); i++) {
						final Tab tab = (Tab) newBar.mContainer.getChildAt(i);
						final PopMenuItem item = menu.items.get(i);
						final String submenuPrefix = mBrowser.application.resources.prefix + ":menu?id=";
						if (item.id != null && item.id.startsWith(submenuPrefix)) {
							final PopMenu subMenu = mBrowser.menu.menus.get(item.id.substring(submenuPrefix.length()));
							if (subMenu != null) {
								tab.extraRunnable = new Runnable() {
									@Override
									public void run() {
										try {
											newBar.selectTab(tab.getDivId());
										} catch (Exception e) {
											mBrowser.exception(getInterfaceName() + ".extraRunnable", e);
										}
									}
								};
								subMenu.dismissRunnable = new Runnable() {
									@Override
									public void run() {
										try {
											newBar.selectTab(mBrowser.history.getCurrentItem().getUrl());
										} catch (Exception e) {
											mBrowser.exception(getInterfaceName() + ".dismissRunnable", e);
										}
									}
								};
								if (subMenu.styleSheet.window_type.equalsIgnoreCase("popup")) {
									subMenu.dynamicLocation = new PopMenu.DynamicLocation() {
										@Override
										public int[] run() {
											try {
												int[] location = new int[2];
												tab.getLocationInWindow(location);
												Rect rect = mLayout.getRect();
												location[1] = location[1] - rect.top;
												if (newBar.styleSheet.bar_position.equalsIgnoreCase("bottom")) {
													int height = subMenu.styleSheet.getHeight();
													if (height > 0) location[1] = location[1] - height;
												} else {
													location[1] = location[1] + tab.getHeight();
												}
												return location;
											} catch (Exception e) {
												mBrowser.exception(getInterfaceName() + ".dynamicLocation", e);
												return new int[]{0, 0};
											}
										}
									};
								}
							}
						}
					}
					mLayout.addChild(newBar.styleSheet.bar_position, Utilities.getInt(newBar.styleSheet.index, -1), newBar);
					if (selectedTabIds != null && !selectedTabIds.equals("")) newBar.selectTabs(selectedTabIds.split("\\|"));
					mMap.put(menuId, newBar);

				} catch (Exception e) {
					mBrowser.exception(getInterfaceName(), e);
				}
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
		return;
	}
	public void remove() {
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Set<String> keys = mMap.keySet();
				Iterator<String> iterator = keys.iterator();
				while (iterator.hasNext()) {
					TabBar bar = mMap.get(iterator.next());
					mLayout.removeChild(bar.styleSheet.bar_position, bar);
				}
				mMap.clear();
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}
	public void remove(final String id) {
		final TabBar tabBar = mMap.get(id);
		if (tabBar == null) return;
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				mLayout.removeChild(tabBar.styleSheet.bar_position, tabBar);
				mMap.remove(id);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}
	public String getTabIds(String id) {
		final TabBar bar = mMap.get(id);
		if (bar == null) return "";
		String rtn = "";
		for (int i = 0; i < bar.getChildCount(); i++) {
			rtn += (i == 0) ? "" : ",";
			rtn += "\"" + ((Div) bar.getChildAt(i)).getDivId() + "\"";
		}
		return "[" + rtn + "]";
	}
	public void select(final String id) {
		HistoryItem historyItem = mBrowser.history.getCurrentItem();
		if (historyItem != null) {
			select(id, historyItem.getUrl());
		} else {
			select(id, null);
		}
	}
	public void select(final String id, final String tabIds) {
		if (id == null || id.equals("") || tabIds == null || tabIds.equals("")) return;
		final TabBar bar = mMap.get(id);
		if (bar == null) return;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				bar.selectTabs(tabIds.split("\\|"));
			}
		});
	}
	public void selectByIndex(final String id, final int index) {
		final TabBar bar = mMap.get(id);
		if (bar == null) return;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				bar.selectTab(index);
			}
		});
	}
	public void unselect(final String id) {
		if (id == null || id.equals("")) return;
		final TabBar bar = mMap.get(id);
		if (bar == null) return;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				bar.unselectTabs();
			}
		});
	}

	public void setStyle(String id, final String style) {
		final TabBar tabBar = mMap.get(id);
		if (tabBar == null) return;
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				mLayout.removeChild(tabBar.styleSheet.bar_position, tabBar);
				tabBar.setBarStyle(style);
				mLayout.addChild(tabBar.styleSheet.bar_position, Utilities.getInt(tabBar.styleSheet.index, -1), tabBar);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}
	public String getStyle(String id, String name) {
		final TabBar tabBar = mMap.get(id);
		if (tabBar == null) return "";
		String value = tabBar.styleSheet.getStyle(name);
		return (value == null) ? "" : value;
	}
	public String getCurrentStyle(String id, String name) {
		final TabBar tabBar = mMap.get(id);
		if (tabBar == null) return "";
		String value = tabBar.getStyle(name);
		return (value == null) ? "" : value;
	}

	public void startAnimation(String id, final String style) {
		final TabBar bar = mMap.get(id);
		if (bar == null) return;
		bar.animate(mBrowser, style);
	}
	public void stopAnimation(String id) {
		final TabBar bar = mMap.get(id);
		if (bar == null) return;
		bar.stopAnimate();
	}

}
