package rexsee.core.layout;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.JavascriptInterface;
import rexsee.core.widget.Tab;
import rexsee.core.widget.TabBar;
import android.app.Activity;

public class RexseeTabs implements JavascriptInterface {

	public static final String INTERFACE_NAME = "Tabs";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return null;
	}

	private final Activity mActivity;
	private final Browser mBrowser;
	//private final Layout mLayout;
	private final RexseeTabBars mTabBars;
	private boolean syncTag = false;

	public RexseeTabs(Browser browser, RexseeTabBars tabBars) {
		super();
		mBrowser = browser;
		mActivity = (Activity) browser.getContext();
		mTabBars = tabBars;
	}

	private Tab getTab(String id) {
		if (id == null || id.equals("")) return null;
		HashMap<String, TabBar> mMap = mTabBars.getMap();
		Set<String> ids = mMap.keySet();
		Iterator<String> iterator = ids.iterator();
		while (iterator.hasNext()) {
			TabBar bar = mMap.get(iterator.next());
			for (int j = 0; j < bar.mContainer.getChildCount(); j++) {
				Tab tab = (Tab) bar.mContainer.getChildAt(j);
				if (id.equalsIgnoreCase(tab.getDivId())) return tab;
			}
		}
		return null;
	}

	public boolean exists(String id) {
		final Tab tab = getTab(id);
		return (tab == null) ? false : true;
	}
	public boolean isSelected(String id) {
		final Tab tab = getTab(id);
		return (tab == null) ? false : tab.isSelected();
	}
	public void setSelected(String id, final boolean selected) {
		final Tab tab = getTab(id);
		if (tab == null) return;
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				tab.setSelected(selected);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}

	public void setStyle(String id, final String normalStyle, final String pressedStyle, final String selectedStyle) {
		final Tab tab = getTab(id);
		if (tab == null) return;
		syncTag = false;
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				tab.setStyle(normalStyle, pressedStyle, selectedStyle);
				syncTag = true;
			}
		});
		while (!syncTag) {
			try {
				Thread.sleep(20);
			} catch (Exception e) {

			}
		}
	}
	public String getNormalStyle(String id, String name) {
		final Tab tab = getTab(id);
		if (tab == null) return "";
		String value = tab.styleNormal.getStyle(name);
		return (value == null) ? "" : value;
	}
	public String getPressedStyle(String id, String name) {
		final Tab tab = getTab(id);
		if (tab == null) return "";
		String value = tab.stylePressed.getStyle(name);
		return (value == null) ? "" : value;
	}
	public String getSelectedStyle(String id, String name) {
		final Tab tab = getTab(id);
		if (tab == null) return "";
		String value = tab.styleSelected.getStyle(name);
		return (value == null) ? "" : value;
	}
	public String getCurrentStyle(String id, String name) {
		final Tab tab = getTab(id);
		if (tab == null) return "";
		String value = tab.getStyle(name);
		return (value == null) ? "" : value;
	}

	public void startAnimation(String id, final String style) {
		final Tab tab = getTab(id);
		if (tab == null) return;
		tab.animate(mBrowser, style);
	}
	public void stopAnimation(String id) {
		final Tab tab = getTab(id);
		if (tab == null) return;
		tab.stopAnimate();
		tab.styleNormal.parseStyle("animation-type:none;");
		tab.stylePressed.parseStyle("animation-type:none;");
		tab.styleSelected.parseStyle("animation-type:none;");
	}

}
