package rexsee.core.widget;

import android.content.Context;
import android.graphics.Color;
import android.text.Html;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

public class RatingStars extends LinearLayout {
	private final TextView titleText;
	private final LinearLayout starLayout;
	private int rating;
	private final boolean isStar10;
	private final float mDensity;
	private int scale(int pixel) {
		return Math.round(mDensity * pixel);
	}
	public RatingStars(Context context, final String title, final int init, final boolean star10) {
		super(context);
		mDensity = context.getResources().getDisplayMetrics().density;
		isStar10 = star10;
		this.setOrientation(LinearLayout.HORIZONTAL);
		this.setGravity(Gravity.CENTER_VERTICAL);
		titleText = new TextView(context);
		if (title != null && !title.equals("")) {
			titleText.setId(0);
			titleText.setTextSize(18);
			titleText.setPadding(0, scale(5), scale(5), scale(5));
			titleText.setTextColor(Color.WHITE);
			titleText.setText(Html.fromHtml(title));
			this.addView(titleText, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT));
		}
		starLayout = new LinearLayout(context);
		starLayout.setOrientation(LinearLayout.HORIZONTAL);
		starLayout.setGravity(Gravity.CENTER_VERTICAL);
		for (int i = 0; i < ((star10) ? 10 : 5); i++) {
			TextView star = new TextView(context);
			star.setGravity(Gravity.CENTER);
			star.setPadding(scale(2), scale(5), scale(2), scale(5));
			star.setTextSize((star10) ? 22 : 40);
			star.setClickable(true);
			star.setOnTouchListener(new ViewGroup.OnTouchListener() {
				@Override
				public boolean onTouch(View v, MotionEvent event) {
					int a = event.getAction();
					if (a == MotionEvent.ACTION_DOWN || a == MotionEvent.ACTION_MOVE) {
						rating = Math.round((event.getX() + v.getLeft()) / v.getWidth());
						setRating();
					}
					return false;
				}
			});
			starLayout.addView(star, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT, 1));
		}
		this.addView(starLayout, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT));
		rating = init;
		setRating();
	}
	private void setRating() {
		for (int i = 0; i < starLayout.getChildCount(); i++) {
			TextView s = (TextView) starLayout.getChildAt(i);
			if (i < rating) {
				s.setTextColor(Color.YELLOW);
				s.setText("");
			} else {
				s.setTextColor(Color.GRAY);
				s.setText("");
			}
		}
	}
	public int getRating() {
		if (rating < 0) return 0;
		if (isStar10) {
			return (rating > 10) ? 10 : rating;
		} else {
			return (rating > 5) ? 5 : rating;
		}
	}
}
