package rexsee.natives;

import java.util.Calendar;

import rexsee.core.browser.Browser;
import rexsee.core.browser.BrowserDiv;
import rexsee.core.browser.clazz.PromptArgumentsSheet;
import rexsee.core.style.RexseeDrawable;
import rexsee.core.style.StyleSheet;
import rexsee.core.utilities.Escape;
import rexsee.core.utilities.Utilities;
import rexsee.core.widget.Div;
import rexsee.core.widget.LabelDiv;
import rexsee.core.widget.ProgressDiv;
import rexsee.dialog.ImageMultiTouchListener;
import rexsee.dialog.RexseeImageDialog;
import rexsee.ebook.PageView;
import rexsee.multimedia.VideoView;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.graphics.PixelFormat;
import android.graphics.drawable.Drawable;
import android.os.SystemClock;
import android.text.Editable;
import android.text.Html;
import android.text.InputFilter;
import android.text.TextUtils.TruncateAt;
import android.text.TextWatcher;
import android.text.method.ScrollingMovementMethod;
import android.text.util.Linkify;
import android.view.KeyEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.AnalogClock;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.Chronometer;
import android.widget.DatePicker;
import android.widget.DigitalClock;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.Gallery;
import android.widget.GridView;
import android.widget.HorizontalScrollView;
import android.widget.ImageButton;
import android.widget.ImageSwitcher;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RatingBar;
import android.widget.RelativeLayout;
import android.widget.ScrollView;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextSwitcher;
import android.widget.TextView;
import android.widget.TimePicker;
import android.widget.ToggleButton;
import android.widget.ViewAnimator;
import android.widget.ViewFlipper;
import android.widget.ViewSwitcher;
import android.widget.ZoomControls;

public class ViewStyle {

	public static ViewTag create(final Browser browser, final String id, int uniqueId, String type, String style, RexseeJavaView javaView) {

		if (type == null || type.trim().equals("")) return null;

		final Context context = browser.getContext();
		boolean canHaveChild = false;
		View view;

		//
		if (type.equalsIgnoreCase("FrameLayout")) {
			canHaveChild = true;
			view = new FrameLayout(context);
		} else if (type.equalsIgnoreCase("AbsoluteLayout")) {
			canHaveChild = true;
			view = new AbsoluteLayout(context);
		} else if (type.equalsIgnoreCase("RelativeLayout")) {
			canHaveChild = true;
			view = new RelativeLayout(context);
		} else if (type.equalsIgnoreCase("TableLayout")) {
			canHaveChild = true;
			view = new TableLayout(context);
		} else if (type.equalsIgnoreCase("TableRow")) {
			canHaveChild = true;
			view = new TableRow(context);
		} else if (type.equalsIgnoreCase("ScrollView")) {
			canHaveChild = true;
			view = new ScrollView(context) {
				final ScrollView mThis = this;
				@Override
				protected void onScrollChanged(int l, int t, int oldl, int oldt) {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						if (viewTag != null && viewTag.style.event_scroll.equalsIgnoreCase("true")) {
							browser.eventList.run(RexseeJavaView.EVENT_ONJAVAVIEWSCROLL, new String[]{viewTag.id, String.valueOf(l), String.valueOf(t), String.valueOf(oldl), String.valueOf(oldt)});
						}
					} catch (Exception e) {
					}
				}
				@Override
				public int getSolidColor() {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						return (viewTag == null) ? Color.BLACK : Color.parseColor(viewTag.style.fading_edge_y_color);
					} catch (Exception e) {
						return Color.BLACK;
					}
				}
			};
		} else if (type.equalsIgnoreCase("HorizontalScrollView")) {
			canHaveChild = true;
			view = new HorizontalScrollView(context) {
				final HorizontalScrollView mThis = this;
				@Override
				protected void onScrollChanged(int l, int t, int oldl, int oldt) {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						if (viewTag != null && viewTag.style.event_scroll.equalsIgnoreCase("true")) {
							browser.eventList.run(RexseeJavaView.EVENT_ONJAVAVIEWSCROLL, new String[]{viewTag.id, String.valueOf(l), String.valueOf(t), String.valueOf(oldl), String.valueOf(oldt)});
						}
					} catch (Exception e) {
					}
				}
				@Override
				public int getSolidColor() {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						return (viewTag == null) ? Color.BLACK : Color.parseColor(viewTag.style.fading_edge_y_color);
					} catch (Exception e) {
						return Color.BLACK;
					}
				}

			};
		} else if (type.equalsIgnoreCase("LinearLayout")) {
			canHaveChild = true;
			view = new LinearLayout(context);
		} else if (type.equalsIgnoreCase("RadioGroup")) {
			canHaveChild = true;
			view = new RadioGroup(context);
		} else if (type.equalsIgnoreCase("ViewAnimator")) {
			canHaveChild = true;
			view = new ViewAnimator(context);
		} else if (type.equalsIgnoreCase("ViewFlipper")) {
			canHaveChild = true;
			view = new ViewFlipper(context);
		} else if (type.equalsIgnoreCase("ViewSwitcher")) {
			canHaveChild = true;
			view = new ViewSwitcher(context);
			((ViewSwitcher) view).addView(new FrameLayout(context));
			((ViewSwitcher) view).addView(new FrameLayout(context));
		} else if (type.equalsIgnoreCase("Div")) {
			canHaveChild = true;
			view = new Div(context);
		} else if (type.equalsIgnoreCase("Spinner")) {
			canHaveChild = true;
			Spinner spinner = new Spinner(context) {
				final Spinner mThis = this;
				@Override
				public void onClick(DialogInterface dialog, int which) {
					ViewTag tag = (ViewTag) mThis.getTag();
					if (tag != null) {
						if (which != mThis.getSelectedItemPosition()) {
							this.setSelection(which);
							browser.eventList.run(RexseeJavaView.EVENT_ONSPINNERCHANGED, new String[]{tag.id, String.valueOf(which)});
						}
					}
					dialog.dismiss();
				}
				@Override
				public int getSolidColor() {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						return (viewTag == null) ? Color.BLACK : Color.parseColor(viewTag.style.fading_edge_y_color);
					} catch (Exception e) {
						return Color.BLACK;
					}
				}
			};
			view = spinner;
		} else if (type.equalsIgnoreCase("SlidingDrawer")) {
			canHaveChild = true;
			StyleSheet styleSheet = new StyleSheet(browser.getContext()).parseStyle(style);
			boolean vertical = styleSheet.orientation.equalsIgnoreCase("y") || styleSheet.orientation.equalsIgnoreCase("vertical");
			final SlidingDrawer slidingDrawer = new SlidingDrawer(context, vertical);
			slidingDrawer.setOnDrawerCloseListener(new SlidingDrawer.OnDrawerCloseListener() {
				@Override
				public void onDrawerClosed() {
					ViewTag tag = (ViewTag) slidingDrawer.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONSLIDINGDRAWERCHANGED, new String[]{tag.id, "closed"});
				}
			});
			slidingDrawer.setOnDrawerOpenListener(new SlidingDrawer.OnDrawerOpenListener() {
				@Override
				public void onDrawerOpened() {
					ViewTag tag = (ViewTag) slidingDrawer.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONSLIDINGDRAWERCHANGED, new String[]{tag.id, "opened"});
				}
			});
			slidingDrawer.setOnDrawerScrollListener(new SlidingDrawer.OnDrawerScrollListener() {
				@Override
				public void onScrollStarted() {
					ViewTag tag = (ViewTag) slidingDrawer.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONSLIDINGDRAWERCHANGED, new String[]{tag.id, "scrollstarted"});
				}
				@Override
				public void onScrollEnded() {
					ViewTag tag = (ViewTag) slidingDrawer.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONSLIDINGDRAWERCHANGED, new String[]{tag.id, "ended"});
				}
			});
			view = slidingDrawer;
		} else if (type.equalsIgnoreCase("ListView")) {
			canHaveChild = true;
			final ListView listView = new ListView(context) {
				ListView mThis = this;
				@Override
				protected void onScrollChanged(int l, int t, int oldl, int oldt) {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						if (viewTag != null && viewTag.style.event_scroll.equalsIgnoreCase("true")) {
							browser.eventList.run(RexseeJavaView.EVENT_ONJAVAVIEWSCROLL, new String[]{viewTag.id, String.valueOf(l), String.valueOf(t), String.valueOf(oldl), String.valueOf(oldt)});
						}
					} catch (Exception e) {
					}
				}
				@Override
				public int getSolidColor() {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						return (viewTag == null) ? Color.BLACK : Color.parseColor(viewTag.style.fading_edge_y_color);
					} catch (Exception e) {
						return Color.BLACK;
					}
				}
			};
			listView.setOnItemClickListener(new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
					ViewTag tag = (ViewTag) listView.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONITEMCLICKED, new String[]{tag.id, String.valueOf(position)});
				}
			});
			listView.setOnItemLongClickListener(new OnItemLongClickListener() {
				@Override
				public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
					ViewTag tag = (ViewTag) listView.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONITEMLONGCLICKED, new String[]{tag.id, String.valueOf(position)});
					return false;
				}
			});
			view = listView;
		} else if (type.equalsIgnoreCase("GridView")) {
			canHaveChild = true;
			final GridView gridView = new GridView(context) {
				GridView mThis = this;
				@Override
				protected void onScrollChanged(int l, int t, int oldl, int oldt) {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						if (viewTag != null && viewTag.style.event_scroll.equalsIgnoreCase("true")) {
							browser.eventList.run(RexseeJavaView.EVENT_ONJAVAVIEWSCROLL, new String[]{viewTag.id, String.valueOf(l), String.valueOf(t), String.valueOf(oldl), String.valueOf(oldt)});
						}
					} catch (Exception e) {
					}
				}
				@Override
				public int getSolidColor() {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						return (viewTag == null) ? Color.BLACK : Color.parseColor(viewTag.style.fading_edge_y_color);
					} catch (Exception e) {
						return Color.BLACK;
					}
				}
			};
			gridView.setOnItemClickListener(new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
					ViewTag tag = (ViewTag) gridView.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONITEMCLICKED, new String[]{tag.id, String.valueOf(position)});
				}
			});
			gridView.setOnItemLongClickListener(new OnItemLongClickListener() {
				@Override
				public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
					ViewTag tag = (ViewTag) gridView.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONITEMLONGCLICKED, new String[]{tag.id, String.valueOf(position)});
					return false;
				}
			});
			view = gridView;
		} else if (type.equalsIgnoreCase("Gallery")) {
			canHaveChild = true;
			Gallery gallery = new Gallery(context) {
				Gallery mThis = this;
				@Override
				protected void onScrollChanged(int l, int t, int oldl, int oldt) {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						if (viewTag != null && viewTag.style.event_scroll.equalsIgnoreCase("true")) {
							browser.eventList.run(RexseeJavaView.EVENT_ONJAVAVIEWSCROLL, new String[]{viewTag.id, String.valueOf(l), String.valueOf(t), String.valueOf(oldl), String.valueOf(oldt)});
						}
					} catch (Exception e) {
					}
				}
				@Override
				public int getSolidColor() {
					try {
						ViewTag viewTag = (ViewTag) mThis.getTag();
						return (viewTag == null) ? Color.BLACK : Color.parseColor(viewTag.style.fading_edge_x_color);
					} catch (Exception e) {
						return Color.BLACK;
					}
				}
			};
			gallery.setOnItemSelectedListener(new GalleryOnItemSelectedListener(browser, gallery));
			view = gallery;
		} else if (type.equalsIgnoreCase("ThreeDGallery")) {
			canHaveChild = true;
			CustomizedGallery gallery = new CustomizedGallery(context);
			gallery.setOnItemSelectedListener(new GalleryOnItemSelectedListener(browser, gallery));
			view = gallery;
		} else if (type.equalsIgnoreCase("PageView")) {
			canHaveChild = true;
			System.gc();
			StyleSheet styleSheet = new StyleSheet(browser.getContext()).parseStyle(style);
			Drawable pageBack = RexseeDrawable.getDrawable(browser, styleSheet.background_color);
			Drawable background = RexseeDrawable.getDrawable(browser, styleSheet.label_background_color);
			view = new PageView(browser, id, style, null, null, pageBack, background);
		}

		//
		else if (type.equalsIgnoreCase("ImageView")) view = new ImageView(context);
		else if (type.equalsIgnoreCase("ImageButton")) view = new ImageButton(context);
		else if (type.equalsIgnoreCase("TextView")) view = new TextView(context);
		else if (type.equalsIgnoreCase("EditText")) {
			final EditText editText = new EditText(context);
			editText.addTextChangedListener(new TextWatcher() {
				@Override
				public void afterTextChanged(Editable s) {
					ViewTag tag = (ViewTag) editText.getTag();
					if (tag != null) {
						browser.eventList.run(RexseeJavaView.EVENT_ONEDITTEXTCHANGED, new String[]{tag.id, Escape.escape(((TextView) editText).getText().toString())});
					}
				}
				@Override
				public void beforeTextChanged(CharSequence s, int start, int count, int after) {
				}
				@Override
				public void onTextChanged(CharSequence s, int start, int before, int count) {
				}
			});
			view = editText;
		} else if (type.equalsIgnoreCase("Button")) view = new Button(context);
		else if (type.equalsIgnoreCase("ToggleButton")) view = new ToggleButton(context);
		else if (type.equalsIgnoreCase("RadioButton")) view = new RadioButton(context);
		else if (type.equalsIgnoreCase("CheckBox")) view = new CheckBox(context);
		else if (type.equalsIgnoreCase("DigitalClock")) view = new DigitalClock(context);
		else if (type.equalsIgnoreCase("AnalogClock")) view = new AnalogClock(context);
		else if (type.equalsIgnoreCase("WebView") || type.equalsIgnoreCase("Browser") || type.equalsIgnoreCase("BrowserDiv")) view = new BrowserDiv(context, browser, null);
		else if (type.equalsIgnoreCase("Label") || type.equalsIgnoreCase("LabelDiv")) view = new LabelDiv(context);
		else if (type.equalsIgnoreCase("Progress") || type.equalsIgnoreCase("ProgressDiv")) view = new ProgressDiv(context);
		else if (type.equalsIgnoreCase("ProgressBar")) {
			StyleSheet styleSheet = new StyleSheet(browser.getContext()).parseStyle(style);
			ProgressBar progressBar;
			if (styleSheet.mode.equalsIgnoreCase("circular")) {
				progressBar = new ProgressBar(context);
			} else {
				progressBar = new ProgressBar(context, null, android.R.attr.progressBarStyleHorizontal);
				progressBar.setMax(100);
			}
			view = progressBar;
		} else if (type.equalsIgnoreCase("SeekBar")) {
			SeekBar seekBar = new SeekBar(context);
			seekBar.setMax(100);
			seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
				@Override
				public void onStopTrackingTouch(SeekBar seekBar) {
				}
				@Override
				public void onStartTrackingTouch(SeekBar seekBar) {
				}
				@Override
				public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
					if (fromUser) {
						ViewTag tag = (ViewTag) seekBar.getTag();
						if (tag != null) {
							tag.style.progress = progress + "%";
							browser.eventList.run(RexseeJavaView.EVENT_ONSEEKBARCHANGED, new String[]{tag.id, String.valueOf(progress)});
						}
					}
				}
			});
			view = seekBar;
		} else if (type.equalsIgnoreCase("RatingBar")) {
			RatingBar ratingBar = new RatingBar(context);
			ratingBar.setMax(100);
			ratingBar.setIsIndicator(false);
			ratingBar.setOnRatingBarChangeListener(new RatingBar.OnRatingBarChangeListener() {
				@Override
				public void onRatingChanged(RatingBar ratingBar, float rating, boolean fromUser) {
					if (fromUser) {
						ViewTag tag = (ViewTag) ratingBar.getTag();
						if (tag != null) {
							browser.eventList.run(RexseeJavaView.EVENT_ONRATINGBARCHANGED, new String[]{tag.id, String.valueOf(rating)});
						}
					}
				}
			});
			view = ratingBar;
		} else if (type.equalsIgnoreCase("DatePicker")) {
			DatePicker datePicker = new DatePicker(context);
			Calendar rightNow = Calendar.getInstance();
			datePicker.init(rightNow.get(Calendar.YEAR), rightNow.get(Calendar.MONTH), rightNow.get(Calendar.DATE), new DatePicker.OnDateChangedListener() {
				@Override
				public void onDateChanged(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
					ViewTag tag = (ViewTag) view.getTag();
					if (tag != null) {
						browser.eventList.run(RexseeJavaView.EVENT_ONDATEPICKERCHANGED, new String[]{tag.id, String.valueOf(year), String.valueOf(monthOfYear + 1), String.valueOf(dayOfMonth)});
					}
				}
			});
			view = datePicker;
		} else if (type.equalsIgnoreCase("TimePicker")) {
			TimePicker timePicker = new TimePicker(context);
			Calendar rightNow = Calendar.getInstance();
			timePicker.setCurrentHour(rightNow.get(Calendar.HOUR_OF_DAY));
			timePicker.setCurrentMinute(rightNow.get(Calendar.MINUTE));
			timePicker.setIs24HourView(true);
			timePicker.setOnTimeChangedListener(new TimePicker.OnTimeChangedListener() {
				@Override
				public void onTimeChanged(TimePicker view, int hourOfDay, int minute) {
					ViewTag tag = (ViewTag) view.getTag();
					if (tag != null) {
						browser.eventList.run(RexseeJavaView.EVENT_ONTIMEPICKERCHANGED, new String[]{tag.id, String.valueOf(hourOfDay), String.valueOf(minute)});
					}
				}
			});
			view = timePicker;
		} else if (type.equalsIgnoreCase("ZoomControls")) {
			final ZoomControls zoomControls = new ZoomControls(context);
			zoomControls.setOnZoomInClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					ViewTag tag = (ViewTag) zoomControls.getTag();
					if (tag != null) {
						browser.eventList.run(RexseeJavaView.EVENT_ONZOOMCONTROLPRESSED, new String[]{tag.id, "zoomIn"});
					}
				}
			});
			zoomControls.setOnZoomOutClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					ViewTag tag = (ViewTag) zoomControls.getTag();
					if (tag != null) {
						browser.eventList.run(RexseeJavaView.EVENT_ONZOOMCONTROLPRESSED, new String[]{tag.id, "zoomOut"});
					}
				}
			});
			view = zoomControls;
		} else if (type.equalsIgnoreCase("VideoView")) {
			view = new VideoView(browser, id, "", true, false);
		} else if (type.equalsIgnoreCase("ImageSwitcher")) {
			view = new ImageSwitcher(context);
			((ImageSwitcher) view).addView(new ImageView(context));
			((ImageSwitcher) view).addView(new ImageView(context));
		} else if (type.equalsIgnoreCase("TextSwitcher")) {
			view = new TextSwitcher(context);
			((TextSwitcher) view).addView(new TextView(context));
			((TextSwitcher) view).addView(new TextView(context));
		} else if (type.equalsIgnoreCase("SurfaceView")) {
			final SurfaceView surfaceView = new SurfaceView(context);
			SurfaceHolder holder = (surfaceView).getHolder();
			holder.addCallback(new SurfaceHolder.Callback() {
				@Override
				public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
					ViewTag tag = (ViewTag) surfaceView.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONSURFACECHANGED, new String[]{tag.id, "resized", String.valueOf(format), String.valueOf(width), String.valueOf(height)});
				}
				@Override
				public void surfaceCreated(SurfaceHolder holder) {
					ViewTag tag = (ViewTag) surfaceView.getTag();
					if (tag != null) {
						browser.eventList.run(RexseeJavaView.EVENT_ONSURFACECHANGED, new String[]{tag.id, "created", "", "", ""});
						if (tag.resumeRunnable != null) tag.resumeRunnable.run();
					}
				}
				@Override
				public void surfaceDestroyed(SurfaceHolder holder) {
					ViewTag tag = (ViewTag) surfaceView.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONSURFACECHANGED, new String[]{tag.id, "destroyed", "", "", ""});
				}
			});
			holder.setFixedSize(FrameLayout.LayoutParams.FILL_PARENT, FrameLayout.LayoutParams.FILL_PARENT);
			holder.setFormat(PixelFormat.TRANSLUCENT);
			surfaceView.setBackgroundColor(Color.TRANSPARENT);
			surfaceView.setZOrderOnTop(true);
			view = surfaceView;
		} else if (type.equalsIgnoreCase("Chronometer")) {
			final Chronometer meter = new Chronometer(context);
			meter.setOnChronometerTickListener(new Chronometer.OnChronometerTickListener() {
				@Override
				public void onChronometerTick(Chronometer chronometer) {
					ViewTag tag = (ViewTag) chronometer.getTag();
					if (tag != null) browser.eventList.run(RexseeJavaView.EVENT_ONCHRONOMETERCHANGED, new String[]{tag.id, chronometer.getText().toString()});
				}
			});
			view = meter;
		}

		/* Useless: 
		 * else if (type.equalsIgnoreCase("ZoomButton")) view = new ZoomButton(context);
		*/

		else return null;

		try {
			view.setId(uniqueId);
			view.setOnKeyListener(new View.OnKeyListener() {
				@Override
				public boolean onKey(View v, int keyCode, KeyEvent event) {
					browser.onKeyDown(keyCode, event);
					return false;
				}
			});
			ViewTag viewTag = new ViewTag(context, id, uniqueId, type, view, canHaveChild);
			view.setTag(viewTag);
			if (!style.equals("debug")) setStyle(browser, viewTag, style, javaView);
			return viewTag;
		} catch (Exception e) {
			browser.exception("Native-view Creation", e);
			return null;
		}

	}

	public static void setStyle(Browser browser, ViewTag viewTag, String style, RexseeJavaView javaView) {

		if (viewTag.type.equalsIgnoreCase("Button")) style = "min-height:24;width:wrapcontent;height:wrapcontent;label-padding:0 20 2 20;label-background-color:default;background-color:default;color:#000000;" + style;
		else if (viewTag.type.equalsIgnoreCase("ImageButton")) style = "width:wrapcontent;height:wrapcontent;icon-padding:10 10 15 10;background-color:default;icon-background-color:default;" + style;
		else if (viewTag.type.equalsIgnoreCase("ToggleButton")) style = "width:100;height:60;label-padding-bottom:15px;label-background-color:default;background-color:default;color:#000000;" + style;
		else if (viewTag.type.equalsIgnoreCase("EditText")) style = "min-height:24;height:wrapcontent;label-padding:3 10 5 10;label-background-color:default;background-color:default;color:#000000;text-align:left;" + style;
		else if (viewTag.type.equalsIgnoreCase("RadioButton")) style = "min-height:40;height:wrapcontent;label-padding:8 5 8 40;text-align:left;" + style;
		else if (viewTag.type.equalsIgnoreCase("CheckBox")) style = "min-height:40;height:wrapcontent;label-padding:8 5 8 40;text-align:left;" + style;
		else if (viewTag.type.equalsIgnoreCase("Chronometer")) style = "duration:0;" + style;
		else if (viewTag.type.equalsIgnoreCase("ProgressBar")) style = "label-background-color:default;background-color:default;progress-color:default;height:wrapcontent;" + style;
		else if (viewTag.type.equalsIgnoreCase("SeekBar")) style = "label-background-color:default;background-color:default;progress-color:default;" + style + ";height:wrapcontent;";
		else if (viewTag.type.equalsIgnoreCase("RatingBar")) style = "label-background-color:default;background-color:default;progress-color:default;" + style + ";width:wrapcontent;height:wrapcontent;";
		else if (viewTag.type.equalsIgnoreCase("Spinner")) style = "width:wrapcontent;height:wrapcontent;label-background-color:default;background-color:default;color:black;font-size:16px;padding:6 55 10 15;label-padding:0;" + style;
		else if (viewTag.type.equalsIgnoreCase("DatePicker")) style = "width:wrapcontent;height:wrapcontent;" + style;
		else if (viewTag.type.equalsIgnoreCase("TimePicker")) style = "width:wrapcontent;height:wrapcontent;" + style;
		else if (viewTag.type.equalsIgnoreCase("ZoomControls")) style = "width:wrapcontent;height:wrapcontent;" + style;
		else if (viewTag.type.equalsIgnoreCase("Gallery") || viewTag.type.equalsIgnoreCase("ThreeDGallery")) style = "width:fillparent;height:fillparent;icon-height:fillparent;icon-width:fillparent;background-color:#000000;icon-background-color:default;label-background-color:default;icon-alpha:0.9;horizontal-spacing:0;duration:200;" + style;
		else if (viewTag.type.equalsIgnoreCase("SurfaceView")) style = "width:fillparent;height:fillparent;background-color:transparent;format:default;mode:ZOrderOnTop;" + style;
		else if (viewTag.type.equalsIgnoreCase("ListView")) style = "horizontal-line:#888888;vertical-spacing:1;" + style;
		else if (viewTag.type.equalsIgnoreCase("ViewFlipper")) style = "duration:2000;" + style;

		viewTag.style.parseStyle(style);

		if (viewTag.type.equalsIgnoreCase("WebView") || viewTag.type.equalsIgnoreCase("Browser") || viewTag.type.equalsIgnoreCase("BrowserDiv")) {
			((BrowserDiv) viewTag.view).setStyle(style);
		} else if (viewTag.type.equalsIgnoreCase("Div")) {
			((Div) viewTag.view).setStyle(style);
			if (viewTag.style.event_touch.equalsIgnoreCase("true") && !viewTag.touchEvent) {
				viewTag.touchEvent = true;
				viewTag.view.setClickable(true);
				viewTag.view.setOnTouchListener(new ViewOnTouchListener(browser, viewTag.id, viewTag.style.event_touchmove.equalsIgnoreCase("true"), null));
			} else if (viewTag.style.event_touch.equalsIgnoreCase("false") && viewTag.touchEvent) {
				viewTag.touchEvent = true;
				viewTag.view.setClickable(false);
				viewTag.view.setOnTouchListener(null);
			}
		} else if (viewTag.type.equalsIgnoreCase("Label") || viewTag.type.equalsIgnoreCase("LabelDiv")) {
			((LabelDiv) viewTag.view).setStyle(style);
			if (viewTag.style.event_touch.equalsIgnoreCase("true") && !viewTag.touchEvent) {
				viewTag.touchEvent = true;
				viewTag.view.setClickable(true);
				viewTag.view.setOnTouchListener(new ViewOnTouchListener(browser, viewTag.id, viewTag.style.event_touchmove.equalsIgnoreCase("true"), null));
			} else if (viewTag.style.event_touch.equalsIgnoreCase("false") && viewTag.touchEvent) {
				viewTag.touchEvent = true;
				viewTag.view.setClickable(false);
				viewTag.view.setOnTouchListener(null);
			}
		} else if (viewTag.type.equalsIgnoreCase("Progress") || viewTag.type.equalsIgnoreCase("ProgressDiv")) {
			((ProgressDiv) viewTag.view).setStyle(style);
			if (viewTag.style.event_touch.equalsIgnoreCase("true") && !viewTag.touchEvent) {
				viewTag.touchEvent = true;
				viewTag.view.setClickable(true);
				viewTag.view.setOnTouchListener(new ViewOnTouchListener(browser, viewTag.id, viewTag.style.event_touchmove.equalsIgnoreCase("true"), null));
			} else if (viewTag.style.event_touch.equalsIgnoreCase("false") && viewTag.touchEvent) {
				viewTag.touchEvent = true;
				viewTag.view.setClickable(false);
				viewTag.view.setOnTouchListener(null);
			}
		} else {
			ViewStyle.setViewStyle(browser, viewTag.view, viewTag.style);
		}

		if (viewTag.style.position.equalsIgnoreCase("absolute")) {
			viewTag.view.setMinimumWidth(Utilities.getInt(viewTag.style.min_width, 0));
			viewTag.view.setMinimumHeight(Utilities.getInt(viewTag.style.min_height, 0));
			viewTag.view.setLayoutParams(new AbsoluteLayout.LayoutParams(viewTag.style.getWidth(), viewTag.style.getHeight(), viewTag.style.getLeft(), viewTag.style.getTop()));
		} else if (viewTag.style.position.equalsIgnoreCase("relative")) {
			viewTag.view.setMinimumWidth(Utilities.getInt(viewTag.style.min_width, 0));
			viewTag.view.setMinimumHeight(Utilities.getInt(viewTag.style.min_height, 0));
			viewTag.view.setLayoutParams(getRelativeLayoutParams(javaView, viewTag.style.getWidth(), viewTag.style.getHeight(), viewTag.style.left, viewTag.style.top));
		} else if (viewTag.style.position.equalsIgnoreCase("table")) {
			viewTag.view.setMinimumWidth(Utilities.getInt(viewTag.style.min_width, 0));
			viewTag.view.setMinimumHeight(Utilities.getInt(viewTag.style.min_height, 0));
			TableRow.LayoutParams layoutParams = new TableRow.LayoutParams(viewTag.style.getWidth(), viewTag.style.getHeight());
			int index = Utilities.getInt(viewTag.style.index, -1);
			if (index >= 0) layoutParams.column = index;
			int span = Utilities.getInt(viewTag.style.column_span, 0);
			if (span > 1) layoutParams.span = span;
			viewTag.view.setLayoutParams(layoutParams);
		} else {
			if (viewTag.view.getParent() == null) {
				viewTag.view.setMinimumWidth(Utilities.getInt(viewTag.style.min_width, 0));
				viewTag.view.setMinimumHeight(Utilities.getInt(viewTag.style.min_height, 0));
				viewTag.view.setLayoutParams(viewTag.style.getLayoutParams());
			}
		}

		if (viewTag.type.equalsIgnoreCase("TextView") || viewTag.type.equalsIgnoreCase("EditText") || viewTag.type.equalsIgnoreCase("Button") || viewTag.type.equalsIgnoreCase("RadioButton") || viewTag.type.equalsIgnoreCase("CheckBox") || viewTag.type.equalsIgnoreCase("ToggleButton") || viewTag.type.equalsIgnoreCase("Chronometer") || viewTag.type.equalsIgnoreCase("DigitalClock")) ViewStyle.setTextViewStyle(browser, (TextView) viewTag.view, viewTag.style);
		if (viewTag.type.equalsIgnoreCase("EditText")) setEditTextStyle(browser, (EditText) viewTag.view, viewTag.style);
		if (viewTag.type.equalsIgnoreCase("ImageView") || viewTag.type.equalsIgnoreCase("ImageButton")) ViewStyle.setImageViewStyle(browser, (ImageView) viewTag.view, viewTag.style);
		if (viewTag.type.equalsIgnoreCase("SurfaceView")) ViewStyle.setSurfaceViewStyle(browser, (SurfaceView) viewTag.view, viewTag.style);

		if (viewTag.type.equalsIgnoreCase("VideoView")) {
			try {
				VideoView videoView = (VideoView) viewTag.view;
				videoView.setZOrderOnTop(true);
				videoView.isLooping = viewTag.style.mode.equalsIgnoreCase("loop") || viewTag.style.mode.equalsIgnoreCase("looping");
				videoView.isKeepScale = viewTag.style.format.equalsIgnoreCase("fixscale");
				if (!viewTag.style.path.trim().equals(videoView.path.trim())) {
					//if (videoView.mediaPlayer != null) videoView.mediaPlayer.stop();
					videoView.path = viewTag.style.path;
					//videoView.openVideo();
				}
			} catch (Exception e) {
				browser.exception("SetStyle/VideoView", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("LinearLayout") || viewTag.type.equalsIgnoreCase("RadioGroup")) {
			try {
				LinearLayout linearLayout = (LinearLayout) viewTag.view;
				linearLayout.setOrientation(viewTag.style.orientation.equalsIgnoreCase("x") || viewTag.style.orientation.equalsIgnoreCase("horizontal") ? LinearLayout.HORIZONTAL : LinearLayout.VERTICAL);
				linearLayout.setGravity(viewTag.style.getGravity());
			} catch (Exception e) {
				browser.exception("SetStyle/LinearLayout", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("Chronometer")) {
			try {
				Chronometer chronometer = (Chronometer) viewTag.view;
				if (!viewTag.style.format.trim().equals("")) chronometer.setFormat(viewTag.style.format);
				chronometer.setBase(SystemClock.elapsedRealtime() + Utilities.getInt(viewTag.style.duration, 0));
			} catch (Exception e) {
				browser.exception("SetStyle/Chronometer", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("ProgressBar")) {
			try {
				ProgressBar progressBar = (ProgressBar) viewTag.view;
				Drawable progress = RexseeDrawable.getDrawable(browser, viewTag.style.progress_color, null);
				if (viewTag.style.mode.equals("indetermination")) {
					progressBar.setIndeterminate(true);
					if (progress != null) progressBar.setIndeterminateDrawable(progress);
				} else {
					progressBar.setIndeterminate(false);
					if (progress != null) progressBar.setProgressDrawable(progress);
					int p = Utilities.getInt(viewTag.style.progress.replace("%", "").trim(), 0);
					p = (p < 0) ? 0 : p;
					p = (p > 100) ? 100 : p;
					progressBar.setProgress(p);
				}
			} catch (Exception e) {
				browser.exception("SetStyle/ProgressBar", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("SeekBar")) {
			try {
				SeekBar seekBar = (SeekBar) viewTag.view;
				Drawable progress = RexseeDrawable.getDrawable(browser, viewTag.style.progress_color, null);
				if (progress != null) seekBar.setProgressDrawable(progress);
				int p = Utilities.getInt(viewTag.style.progress.replace("%", "").trim(), 0);
				p = (p < 0) ? 0 : p;
				p = (p > 100) ? 100 : p;
				seekBar.setProgress(p);
			} catch (Exception e) {
				browser.exception("SetStyle/SeekBar", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("RatingBar")) {
			try {
				RatingBar ratingBar = (RatingBar) viewTag.view;
				Drawable progress = RexseeDrawable.getDrawable(browser, viewTag.style.progress_color, null);
				if (progress != null) ratingBar.setProgressDrawable(progress);
				ratingBar.setNumStars(Utilities.getInt(viewTag.style.rating_stars.trim(), 5));
				ratingBar.setStepSize(Utilities.getFloat(viewTag.style.rating_step.trim(), 0.5f));
				ratingBar.setIsIndicator(viewTag.style.mode.equalsIgnoreCase("indicator"));
			} catch (Exception e) {
				browser.exception("SetStyle/RatingBar", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("Spinner")) {
			try {
				Spinner spinner = (Spinner) viewTag.view;
				if (!viewTag.style.label_type.trim().equals("")) {
					if (viewTag.style.label_type.equalsIgnoreCase("html")) {
						spinner.setPrompt(Html.fromHtml(viewTag.style.label));
					} else if (viewTag.style.label_type.equalsIgnoreCase("string")) {
						spinner.setPrompt(Html.fromHtml(viewTag.style.label).toString());
					} else {
						spinner.setPrompt(viewTag.style.label);
					}
				}
			} catch (Exception e) {
				browser.exception("SetStyle/Spinner", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("Gallery") || viewTag.type.equalsIgnoreCase("ThreeDGallery")) {
			try {
				Gallery gallery = (Gallery) viewTag.view;
				gallery.setGravity(viewTag.style.getGravity());
				gallery.setUnselectedAlpha(viewTag.style.getIconAlpha());
				gallery.setAnimationDuration(Utilities.getInt(viewTag.style.duration, 200));
				gallery.setSpacing(viewTag.style.getHorizontalSpacing());
			} catch (Exception e) {
				browser.exception("SetStyle/Gallery", e);
			}
		}

		if (viewTag.type.equalsIgnoreCase("ImageSwitcher")) {
			setImageViewStyle(browser, (ImageView) ((ImageSwitcher) viewTag.view).getChildAt(0), viewTag.style);
			setImageViewStyle(browser, (ImageView) ((ImageSwitcher) viewTag.view).getChildAt(1), viewTag.style);
		}
		if (viewTag.type.equalsIgnoreCase("TextSwitcher")) {
			setTextViewStyle(browser, (TextView) ((TextSwitcher) viewTag.view).getChildAt(0), viewTag.style);
			setTextViewStyle(browser, (TextView) ((TextSwitcher) viewTag.view).getChildAt(1), viewTag.style);
		}
		if (viewTag.type.equalsIgnoreCase("ViewFlipper")) {
			try {
				((ViewFlipper) viewTag.view).setFlipInterval(Utilities.getInt(viewTag.style.duration, 2000));
			} catch (Exception e) {
				browser.exception("SetStyle/In and out animation.", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("ListView")) {
			try {
				ListView listView = (ListView) viewTag.view;
				Drawable divider = RexseeDrawable.getDrawable(browser, viewTag.style.horizontal_line);
				if (divider != null) {
					listView.setDivider(divider);
					listView.setDividerHeight(viewTag.style.getVerticalSpacing());
				}
			} catch (Exception e) {
				browser.exception("SetStyle/ListView", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("GridView")) {
			try {
				GridView gridView = (GridView) viewTag.view;
				gridView.setGravity(viewTag.style.getGravity());
				int stretch;
				if (viewTag.style.format.equalsIgnoreCase("STRETCH_SPACING")) {
					stretch = GridView.STRETCH_SPACING;
				} else if (viewTag.style.format.equalsIgnoreCase("STRETCH_SPACING_UNIFORM")) {
					stretch = GridView.STRETCH_SPACING_UNIFORM;
				} else if (viewTag.style.format.equalsIgnoreCase("STRETCH_COLUMN_WIDTH")) {
					stretch = GridView.STRETCH_COLUMN_WIDTH;
				} else {
					stretch = GridView.NO_STRETCH;
				}
				gridView.setStretchMode(stretch);
				int numOfColumns = Utilities.getInt(viewTag.style.column_number, 0);
				gridView.setNumColumns((numOfColumns > 0) ? numOfColumns : GridView.AUTO_FIT);
				gridView.setColumnWidth(viewTag.style.getColumnWidth());
				gridView.setHorizontalSpacing(viewTag.style.getHorizontalSpacing());
				gridView.setVerticalSpacing(viewTag.style.getVerticalSpacing());
			} catch (Exception e) {
				browser.exception("SetStyle/GridView", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("DatePicker")) {
			try {
				String[] visibility = viewTag.style.label_visibility.split("\\|");
				if (visibility.length == 3) {
					ViewGroup pickerGroup = (ViewGroup) ((DatePicker) viewTag.view).getChildAt(0);
					pickerGroup.getChildAt(0).setVisibility((visibility[0].equalsIgnoreCase("hidden")) ? View.GONE : View.VISIBLE);
					pickerGroup.getChildAt(1).setVisibility((visibility[1].equalsIgnoreCase("hidden")) ? View.GONE : View.VISIBLE);
					pickerGroup.getChildAt(2).setVisibility((visibility[2].equalsIgnoreCase("hidden")) ? View.GONE : View.VISIBLE);
				}
				String[] enable = viewTag.style.enable.split("\\|");
				if (enable.length == 3) {
					ViewGroup pickerGroup = (ViewGroup) ((DatePicker) viewTag.view).getChildAt(0);
					pickerGroup.getChildAt(0).setEnabled(!enable[0].equalsIgnoreCase("false"));
					pickerGroup.getChildAt(1).setEnabled(!enable[1].equalsIgnoreCase("false"));
					pickerGroup.getChildAt(2).setEnabled(!enable[2].equalsIgnoreCase("false"));
				}
			} catch (Exception e) {
				browser.exception("SetStyle/DatePicker", e);
			}
		}
		if (viewTag.type.equalsIgnoreCase("TimePicker")) {
			try {
				((TimePicker) viewTag.view).setIs24HourView(!viewTag.style.format.equals("12"));
				String[] visibility = viewTag.style.label_visibility.split("\\|");
				if (visibility.length == 2) {
					ViewGroup pickerGroup = (ViewGroup) ((TimePicker) viewTag.view).getChildAt(0);
					pickerGroup.getChildAt(0).setVisibility((visibility[0].equalsIgnoreCase("hidden")) ? View.GONE : View.VISIBLE);
					pickerGroup.getChildAt(1).setVisibility((visibility[1].equalsIgnoreCase("hidden")) ? View.GONE : View.VISIBLE);
				}
				String[] enable = viewTag.style.enable.split("\\|");
				if (enable.length == 2) {
					ViewGroup pickerGroup = (ViewGroup) ((TimePicker) viewTag.view).getChildAt(0);
					pickerGroup.getChildAt(0).setEnabled(!enable[0].equalsIgnoreCase("false"));
					pickerGroup.getChildAt(1).setEnabled(!enable[1].equalsIgnoreCase("false"));
				}
			} catch (Exception e) {
				browser.exception("SetStyle/DatePicker", e);
			}
		}
		viewTag.view.postInvalidate();

	}

	public static void setViewStyle(Browser browser, View view, StyleSheet style) {
		try {
			if (style.visibility.equalsIgnoreCase("hidden") || style.visibility.equalsIgnoreCase("gone")) {
				view.setVisibility(View.GONE);
			} else if (style.visibility.equalsIgnoreCase("invisible")) {
				view.setVisibility(View.INVISIBLE);
			} else {
				view.setVisibility(View.VISIBLE);
			}
			Drawable bg = RexseeDrawable.getDrawable(browser, style.background_color, null);
			if (bg != null) view.setBackgroundDrawable(bg);
			view.setPadding(style.getPaddingLeft(), style.getPaddingTop(), style.getPaddingRight(), style.getPaddingBottom());
			if (style.overflow_x.equalsIgnoreCase("visible")) {
				view.setHorizontalFadingEdgeEnabled(true);
				view.setFadingEdgeLength(Integer.parseInt(style.fading_edge_x_length.toLowerCase().replaceAll("px", "")));
				if (style.scrollbar_x.equalsIgnoreCase("hidden")) {
					view.setHorizontalScrollBarEnabled(false);
				} else {
					view.setHorizontalScrollBarEnabled(true);
					view.setScrollBarStyle(style.getScrollbarXStyle());
					view.setScrollbarFadingEnabled((style.scrollbar_x.equalsIgnoreCase("visible")) ? false : true);
				}
			}
			if (style.overflow_y.equalsIgnoreCase("visible")) {
				view.setVerticalFadingEdgeEnabled(true);
				view.setFadingEdgeLength(Integer.parseInt(style.fading_edge_y_length.toLowerCase().replaceAll("px", "")));
				if (style.scrollbar_y.equalsIgnoreCase("hidden")) {
					view.setVerticalScrollBarEnabled(false);
				} else {
					view.setVerticalScrollBarEnabled(true);
					view.setScrollBarStyle(style.getScrollbarYStyle());
					view.setScrollbarFadingEnabled((style.scrollbar_y.equalsIgnoreCase("visible")) ? false : true);
				}
			}
			view.setSoundEffectsEnabled(style.play_sound_effect.equals("true"));
			view.setEnabled(!style.enable.equalsIgnoreCase("false"));
			try {
				ViewTag viewTag = (ViewTag) view.getTag();
				if (viewTag.type.equalsIgnoreCase("ImageView")) {
				} else if (viewTag.type.equalsIgnoreCase("PageView")) {
					view.setClickable(true);
				} else {
					if (style.event_touch.equalsIgnoreCase("true") && !viewTag.touchEvent) {
						viewTag.touchEvent = true;
						view.setClickable(true);
						view.setOnTouchListener(new ViewOnTouchListener(browser, viewTag.id, style.event_touchmove.equalsIgnoreCase("true"), null));
					} else if (style.event_touch.equalsIgnoreCase("false") && viewTag.touchEvent) {
						viewTag.touchEvent = true;
						view.setClickable(false);
						view.setOnTouchListener(null);
					}
				}
			} catch (Exception e) {
			}
		} catch (Exception e) {
			browser.exception("SetStyle/View", e);
		}
	}
	public static void setTextViewStyle(Browser browser, TextView textView, StyleSheet style) {
		try {
			Drawable bg = RexseeDrawable.getDrawable(browser, style.label_background_color, null);
			if (bg != null) textView.setBackgroundDrawable(bg);
			textView.setPadding(style.getLabelPaddingLeft(), style.getLabelPaddingTop(), style.getLabelPaddingRight(), style.getLabelPaddingBottom());
			textView.setTextSize(Float.parseFloat(style.font_size.toLowerCase().replaceAll("px", "")));
			textView.setTextColor(Color.parseColor(style.color));
			textView.setTypeface(null, style.getTypefaceStyle());
			textView.setGravity(style.getTextGravity());
			textView.setShadowLayer(
						Float.parseFloat(style.text_shadow_radius),
						Float.parseFloat(style.text_shadow_dx.toLowerCase().replaceAll("px", "")),
						Float.parseFloat(style.text_shadow_dy.toLowerCase().replaceAll("px", "")),
						Color.parseColor(style.text_shadow_color));
			textView.setAutoLinkMask((style.text_link.equalsIgnoreCase("autolink")) ? Linkify.ALL : 0);
			if (style.text_overflow.equalsIgnoreCase("marquee")) {
				textView.setFocusable(true);
				textView.setHorizontallyScrolling(true);
				textView.setEllipsize(TruncateAt.MARQUEE);
				textView.setMarqueeRepeatLimit(-1);
				textView.setMovementMethod(ScrollingMovementMethod.getInstance());
			} else if (style.text_overflow.equalsIgnoreCase("start")) {
				textView.setHorizontallyScrolling(false);
				textView.setEllipsize(TruncateAt.START);
			} else if (style.text_overflow.equalsIgnoreCase("end")) {
				textView.setHorizontallyScrolling(false);
				textView.setEllipsize(TruncateAt.END);
			} else if (style.text_overflow.equalsIgnoreCase("middle")) {
				textView.setHorizontallyScrolling(false);
				textView.setEllipsize(TruncateAt.MIDDLE);
			} else {
				textView.setHorizontallyScrolling(false);
			}
			boolean singleLine = (style.white_space.equalsIgnoreCase("nowrap")) ? true : false;
			textView.setSingleLine(singleLine);
			if (!singleLine) {
				float mult = Utilities.getInt(style.line_height.toLowerCase().replaceAll("%", ""), 100) / 100f;
				textView.setLineSpacing(0, mult);
			}
			if (!style.label_type.trim().equals("")) {
				if (style.label_type.equalsIgnoreCase("html")) {
					textView.setText(Html.fromHtml(style.label));
				} else if (style.label_type.equalsIgnoreCase("string")) {
					textView.setText(Html.fromHtml(style.label).toString());
				} else {
					textView.setText(style.label);
				}
			}
		} catch (Exception e) {
			browser.exception("SetStyle/TextView", e);
		}
	}
	public static void setEditTextStyle(Browser browser, EditText editText, StyleSheet style) {
		try {
			if (style.selectall.equalsIgnoreCase("true")) editText.setSelectAllOnFocus(true);
			int maxLength = Utilities.getInt(style.maxlength, 0);
			if (maxLength > 0) {
				InputFilter[] filter = new InputFilter[1];
				filter[0] = new InputFilter.LengthFilter(maxLength);
				editText.setFilters(filter);
			}
			editText.setInputType(PromptArgumentsSheet.getInputType(style.format));
			editText.setHintTextColor(Color.parseColor(style.hint_color));
			editText.setHint(style.hint);
			editText.setSingleLine((style.white_space.equalsIgnoreCase("nowrap")) ? true : false);
		} catch (Exception e) {
			browser.exception("SetStyle/EditText:", e);
		}
	}
	public static void setImageViewStyle(final Browser browser, ImageView imageView, StyleSheet style) {
		try {
			imageView.setPadding(style.getIconPaddingLeft(), style.getIconPaddingTop(), style.getIconPaddingRight(), style.getIconPaddingBottom());
			Drawable bg = RexseeDrawable.getDrawable(browser, style.icon_background_color, null);
			if (bg != null) imageView.setBackgroundDrawable(bg);
			Drawable icon = RexseeDrawable.getDrawable(browser, style.icon_url, null);
			if (icon != null) imageView.setImageDrawable(icon);
			imageView.setAlpha(style.getIconAlpha());
			imageView.setScaleType(style.getIconScaleType());
			try {
				final ViewTag viewTag = (ViewTag) imageView.getTag();
				if (style.event_touch.equalsIgnoreCase("true") && !viewTag.touchEvent) {
					viewTag.touchEvent = true;
					imageView.setClickable(true);
					if (style.event_multitouch.equalsIgnoreCase("true")) {
						(imageView).setScaleType(ImageView.ScaleType.CENTER_INSIDE);
						ImageMultiTouchListener multiTouchListener = new ImageMultiTouchListener(imageView, Utilities.getInt(style.distance_threshold, 0), new ImageMultiTouchListener.OnImageDragedOverThresholdListener() {
							@Override
							public void dragedOverThreshold(int direction) {
								String directionStr;
								switch (direction) {
									case ImageMultiTouchListener.OnImageDragedOverThresholdListener.DIRECTION_UP :
										directionStr = "up";
										break;
									case ImageMultiTouchListener.OnImageDragedOverThresholdListener.DIRECTION_DOWN :
										directionStr = "down";
										break;
									case ImageMultiTouchListener.OnImageDragedOverThresholdListener.DIRECTION_LEFT :
										directionStr = "left";
										break;
									case ImageMultiTouchListener.OnImageDragedOverThresholdListener.DIRECTION_RIGHT :
										directionStr = "right";
										break;
									default :
										directionStr = "unknown";
										break;
								}
								browser.eventList.run(RexseeImageDialog.EVENT_ONIMAGEDRAGEDOVERTHRESHOLD, new String[]{viewTag.id, directionStr});
							}
						});
						imageView.setOnTouchListener(new ViewOnTouchListener(browser, viewTag.id, style.event_touchmove.equalsIgnoreCase("true"), multiTouchListener));
					} else {
						imageView.setOnTouchListener(new ViewOnTouchListener(browser, viewTag.id, style.event_touchmove.equalsIgnoreCase("true"), null));
					}
				} else if (style.event_touch.equalsIgnoreCase("false") && viewTag.touchEvent) {
					viewTag.touchEvent = true;
					imageView.setClickable(false);
					imageView.setOnTouchListener(null);
				}
			} catch (Exception e) {
				//Toast.makeText(browser.getContext(), "SetStyle/ImageView/Event:" + e, Toast.LENGTH_LONG).show();
			}
		} catch (Exception e) {
			browser.exception("SetStyle/ImageView", e);
		}
	}
	public static void setSurfaceViewStyle(Browser browser, SurfaceView surfaceView, StyleSheet style) {
		try {
			if (style.mode.equalsIgnoreCase("ZOrderOnTop")) {
				surfaceView.setZOrderOnTop(true);
				surfaceView.setZOrderMediaOverlay(false);
			} else if (style.mode.equalsIgnoreCase("ZOrderMediaOverlay")) {
				surfaceView.setZOrderOnTop(false);
				surfaceView.setZOrderMediaOverlay(true);
			} else if (style.mode.equalsIgnoreCase("ZOrderOnTopAndMediaOverlay")) {
				surfaceView.setZOrderOnTop(true);
				surfaceView.setZOrderMediaOverlay(true);
			} else {
				surfaceView.setZOrderOnTop(false);
				surfaceView.setZOrderMediaOverlay(false);
			}
			String stringFormat = style.format.trim();
			int format;
			if (stringFormat.equals("") || stringFormat.equals("default")) {
				return;
			} else if (stringFormat.equalsIgnoreCase("A_8")) {
				format = PixelFormat.A_8;
			} else if (stringFormat.equalsIgnoreCase("JPEG")) {
				format = PixelFormat.JPEG;
			} else if (stringFormat.equalsIgnoreCase("L_8")) {
				format = PixelFormat.L_8;
			} else if (stringFormat.equalsIgnoreCase("LA_88")) {
				format = PixelFormat.LA_88;
			} else if (stringFormat.equalsIgnoreCase("OPAQUE")) {
				format = PixelFormat.OPAQUE;
			} else if (stringFormat.equalsIgnoreCase("RGB_332")) {
				format = PixelFormat.RGB_332;
			} else if (stringFormat.equalsIgnoreCase("RGB_565")) {
				format = PixelFormat.RGB_565;
			} else if (stringFormat.equalsIgnoreCase("RGB_888")) {
				format = PixelFormat.RGB_888;
			} else if (stringFormat.equalsIgnoreCase("RGBA_4444")) {
				format = PixelFormat.RGBA_4444;
			} else if (stringFormat.equalsIgnoreCase("RGBA_5551")) {
				format = PixelFormat.RGBA_5551;
			} else if (stringFormat.equalsIgnoreCase("RGBA_8888")) {
				format = PixelFormat.RGBA_8888;
			} else if (stringFormat.equalsIgnoreCase("RGBX_8888")) {
				format = PixelFormat.RGBX_8888;
			} else if (stringFormat.equalsIgnoreCase("TRANSLUCENT")) {
				format = PixelFormat.TRANSLUCENT;
			} else if (stringFormat.equalsIgnoreCase("TRANSPARENT")) {
				format = PixelFormat.TRANSPARENT;
			} else if (stringFormat.equalsIgnoreCase("YCbCr_420_SP") || stringFormat.equalsIgnoreCase("NV21")) {
				format = PixelFormat.YCbCr_420_SP;
			} else if (stringFormat.equalsIgnoreCase("YCbCr_422_I") || stringFormat.equalsIgnoreCase("YUY2")) {
				format = PixelFormat.YCbCr_422_I;
			} else if (stringFormat.equalsIgnoreCase("YCbCr_422_SP") || stringFormat.equalsIgnoreCase("NV16")) {
				format = PixelFormat.YCbCr_422_SP;
			} else {
				return;
			}
			surfaceView.getHolder().setFormat(format);
		} catch (Exception e) {
			browser.exception("SetStyle/LinearLayout", e);
		}
	}
	public static RelativeLayout.LayoutParams getRelativeLayoutParams(RexseeJavaView javaView, int width, int height, String left, String top) {

		RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(width, height);

		String[] verbLeft = left.split("\\/");
		if (verbLeft[0].equalsIgnoreCase("left")) {
			layoutParams.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
		} else if (verbLeft[0].equalsIgnoreCase("right")) {
			layoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
		} else if (verbLeft[0].equalsIgnoreCase("center") || verbLeft[0].equalsIgnoreCase("middle")) {
			layoutParams.addRule(RelativeLayout.CENTER_HORIZONTAL);
		} else if (verbLeft[0].equalsIgnoreCase("leftof")) {
			if (verbLeft.length >= 2 && verbLeft[1] != null) layoutParams.addRule(RelativeLayout.LEFT_OF, javaView.getViewUniqueId(verbLeft[1]));
		} else if (verbLeft[0].equalsIgnoreCase("rightof")) {
			if (verbLeft.length >= 2 && verbLeft[1] != null) layoutParams.addRule(RelativeLayout.RIGHT_OF, javaView.getViewUniqueId(verbLeft[1]));
		} else if (verbLeft[0].equalsIgnoreCase("alignleftof")) {
			if (verbLeft.length >= 2 && verbLeft[1] != null) layoutParams.addRule(RelativeLayout.ALIGN_LEFT, javaView.getViewUniqueId(verbLeft[1]));
		} else if (verbLeft[0].equalsIgnoreCase("alignrightof")) {
			if (verbLeft.length >= 2 && verbLeft[1] != null) layoutParams.addRule(RelativeLayout.ALIGN_RIGHT, javaView.getViewUniqueId(verbLeft[1]));
		}

		String[] verbTop = top.split("\\/");
		if (verbTop[0].equalsIgnoreCase("top")) {
			layoutParams.addRule(RelativeLayout.ALIGN_PARENT_TOP);
		} else if (verbTop[0].equalsIgnoreCase("bottom")) {
			layoutParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
		} else if (verbTop[0].equalsIgnoreCase("center") || verbTop[0].equalsIgnoreCase("middle")) {
			layoutParams.addRule(RelativeLayout.CENTER_VERTICAL);
		} else if (verbTop[0].equalsIgnoreCase("topof")) {
			if (verbTop.length >= 2 && verbTop[1] != null) layoutParams.addRule(RelativeLayout.ABOVE, javaView.getViewUniqueId(verbTop[1]));
		} else if (verbTop[0].equalsIgnoreCase("bottomof")) {
			if (verbTop.length >= 2 && verbTop[1] != null) layoutParams.addRule(RelativeLayout.BELOW, javaView.getViewUniqueId(verbTop[1]));
		} else if (verbTop[0].equalsIgnoreCase("aligntopof")) {
			if (verbTop.length >= 2 && verbTop[1] != null) layoutParams.addRule(RelativeLayout.ALIGN_TOP, javaView.getViewUniqueId(verbTop[1]));
		} else if (verbTop[0].equalsIgnoreCase("alignbottomof")) {
			if (verbTop.length >= 2 && verbTop[1] != null) layoutParams.addRule(RelativeLayout.ALIGN_BOTTOM, javaView.getViewUniqueId(verbTop[1]));
		}

		return layoutParams;

	}

}
