package rexsee.security;

import java.security.Key;
import java.security.Provider;
import java.security.Provider.Service;
import java.security.Security;
import java.util.Iterator;
import java.util.Set;

import rexsee.core.browser.Browser;
import rexsee.core.browser.clazz.JavascriptInterface;
import rexsee.core.utilities.Base64;
import rexsee.core.utilities.Json;

public class RexseeSecurity implements JavascriptInterface {

	public static final String INTERFACE_NAME = "Security";
	@Override
	public String getInterfaceName() {
		return mBrowser.application.resources.prefix + INTERFACE_NAME;
	}
	@Override
	public JavascriptInterface getInheritInterface(Browser childBrowser) {
		return this;
	}
	@Override
	public JavascriptInterface getNewInterface(Browser childBrowser) {
		return new RexseeSecurity(childBrowser);
	}

	public static final String ALGORITHM_AES = "AES";
	public static final String ALGORITHM_DES = "DES";
	public static final String ALGORITHM_DES3 = "DESEDE";
	public static final String ALGORITHM_IES = "IES";

	public static final String ALGORITHM_RSA = "RSA";
	public static final String ALGORITHM_DSA = "DSA";

	public static String bytes2hex(byte[] arrB) {
		if (arrB == null) return "";
		int iLen = arrB.length;
		StringBuffer sb = new StringBuffer(iLen * 2); // ÿbyteַܱʾַĳ鳤ȵ
		for (int i = 0; i < iLen; i++) {
			int intTmp = arrB[i];
			while (intTmp < 0)
				intTmp = intTmp + 256; // ѸתΪ
			if (intTmp < 16) sb.append("0"); // С0FҪǰ油0
			sb.append(Integer.toString(intTmp, 16));
		}
		return sb.toString();
	}
	public static byte[] hex2bytes(String strIn) {
		if (strIn == null) return null;
		byte[] arrB = strIn.getBytes();
		int iLen = arrB.length;
		byte[] arrOut = new byte[iLen / 2]; // ַʾһֽڣֽ鳤ַȳ2
		for (int i = 0; i < iLen; i = i + 2) {
			String strTmp = new String(arrB, i, 2);
			arrOut[i / 2] = (byte) Integer.parseInt(strTmp, 16);
		}
		return arrOut;
	}
	public static byte[][] bytesSplit(byte[] src, int size) {
		if (src == null || size <= 0) return null;
		boolean flag = (src.length % size == 0);
		int len = (flag) ? src.length / size : src.length / size + 1;
		byte[][] arr = new byte[len][];
		if (flag) {
			for (int i = 0; i < len; i++) {
				byte[] t = new byte[size];
				for (int j = 0; j < size; j++) {
					t[j] = src[i * size + j];
				}
				arr[i] = t;
			}
		} else {
			for (int i = 0; i < len - 1; i++) {
				byte[] t = new byte[size];
				for (int j = 0; j < size; j++) {
					t[j] = src[i * size + j];
				}
				arr[i] = t;
			}
			byte[] tt = new byte[src.length % size];
			for (int k = 0; k < tt.length; k++) {
				tt[k] = src[(len - 1) * size + k];
			}
			arr[len - 1] = tt;
		}
		return arr;
	}

	public static String encodeKey(byte[] keyBytes) {
		//return Base64.encode(keyBytes, Base64.DEFAULT);
		char[] rtn = Base64.encode(keyBytes);
		return new String(rtn);
	}
	public static byte[] decodeKey(String keyString) {
		//return Base64.decode(keyString, Base64.DEFAULT);
		return Base64.decode(keyString);
	}

	public static String key2string_(Key key) {
		return new String(encodeKey(key.getEncoded()));
	}

	private final Browser mBrowser;

	public RexseeSecurity(Browser browser) {
		mBrowser = browser;
	}

	//JavaScript interface

	public String getProviders() {
		Provider[] providers = Security.getProviders();
		String rtn = "";
		for (int i = 0; i < providers.length; i++) {
			if (i > 0) rtn += ",";
			rtn += "{";
			rtn += "\"name\":\"" + providers[i].getName() + "\"";
			rtn += ",\"description\":\"" + providers[i].getInfo() + "\"";
			rtn += ",\"version\":" + String.valueOf(providers[i].getVersion());
			rtn += ",\"services\":[";
			Set<Service> services = providers[i].getServices();
			Iterator<Service> iterator = services.iterator();
			String srv = "";
			while (iterator.hasNext()) {
				if (!srv.equals("")) srv += ",";
				Service s = iterator.next();
				srv += "{";
				srv += "\"algorithm\":\"" + s.getAlgorithm() + "\"";
				srv += ",\"className\":\"" + s.getClassName() + "\"";
				srv += ",\"type\":\"" + s.getType() + "\"";
				srv += "}";
			}
			rtn += srv;
			rtn += "]";
			rtn += "}";
		}
		return "[" + rtn + "]";
	}
	public String getAlgorithms(String type) {
		return Json.toJson(Security.getAlgorithms(type));
	}

	//Class
}
